from steamship import File

from ..client.helpers import deploy_plugin, _steamship
from ..demo_apps.plugin_file_importer import TEST_DOC

__copyright__ = "Steamship"
__license__ = "MIT"


def test_e2e_importer():
    client = _steamship()
    with deploy_plugin("plugin_file_importer.py","fileImporter") as (plugin, version, instance):
        file = File.create(
            client=client,
            name="Test.txt",
            content="This is a test.",
            pluginInstance=instance.handle
        ).data

        data = file.raw().data

        assert (data.decode('utf-8') == TEST_DOC)

        file.delete()



import contextlib
import io
import os
import random
import string
import time
import zipfile
from typing import Dict

from steamship import App, AppVersion, AppInstance
from steamship import Steamship, EmbeddingIndex, File
from steamship.base import Client
from steamship.extension.corpus import Corpus
from steamship.data.plugin import Plugin
from steamship.data.plugin_instance import PluginInstance
from steamship.data.plugin_version import PluginVersion
from steamship.data.user import User

__copyright__ = "Steamship"
__license__ = "MIT"


def _env_or(env_var: str, or_val: str) -> str:
    if env_var in os.environ:
        return os.getenv(env_var)
    return or_val


def _random_name() -> str:
    letters = string.digits + string.ascii_letters
    id = ''.join(random.choice(letters) for i in range(10))
    return "test_{}".format(id)


_TEST_EMBEDDER = "test-embedder-v1"


@contextlib.contextmanager
def _random_index(steamship: Steamship, pluginInstance: str) -> EmbeddingIndex:
    index = steamship.create_index(
        pluginInstance=pluginInstance
    ).data
    yield index
    index.delete()  # or whatever you need to do at exit


@contextlib.contextmanager
def _random_file(steamship: Steamship, content: str = "") -> File:
    file = steamship.create_file(
        name=_random_name(),
        contents=content
    ).data
    yield file
    file.delete()  # or whatever you need to do at exit


@contextlib.contextmanager
def _random_corpus(client: Steamship) -> Corpus:
    corpus = Corpus.create(client, name=_random_name()).data
    yield corpus
    corpus.delete()


def _steamship() -> Steamship:
    # This should automatically pick up variables from the environment.
    return Steamship(profile="test")


def create_app_zip(filename) -> bytes:
    full_path = os.path.join(os.path.dirname(__file__), '..', 'demo_apps', filename)
    zip_buffer = io.BytesIO()

    files = []
    files.append(('api.py', io.BytesIO(open(full_path, 'rb').read())))

    # TODO: This is very dependent on the setup of the local machine.
    # Might be good to find a more machine-invariant solution here.
    # The goal is to copy in all the dependencies of the lambda package.
    # Which are: steamship (current repo), setuptools_scm, requests
    venv = os.path.join(os.path.dirname(__file__), '..', '..', '.venv', 'lib', 'python3.8', 'site-packages')

    pypi_zips = [
        os.path.join(os.path.dirname(__file__), '..', '..', 'src', 'steamship'),
        os.path.join(venv, 'setuptools_scm'),
        os.path.join(venv, 'requests'),
        os.path.join(venv, 'charset_normalizer'),
        os.path.join(venv, 'certifi'),
        os.path.join(venv, 'urllib3'),
        os.path.join(venv, 'idna'),
    ]

    with zipfile.ZipFile(zip_buffer, "a", zipfile.ZIP_DEFLATED, False) as zip_file:
        for file_name, data in files:
            zip_file.writestr(file_name, data.getvalue())

        for pypi_zip in pypi_zips:
            for root, dirs, files in os.walk(pypi_zip):
                for file in files:
                    zip_file.write(
                        os.path.join(root, file),
                        os.path.relpath(
                            os.path.join(root, file),
                            os.path.join(pypi_zip, '..')
                        )
                    )

    ret = zip_buffer.getvalue()

    with open("/tmp/{}.zip".format(filename), "wb") as f:  # use `wb` mode
        f.write(ret)

    return ret


@contextlib.contextmanager
def deploy_app(py_name: str, versionConfigTemplate : Dict[str, any] = None, instanceConfig : Dict[str, any] = None):
    client = _steamship()
    name = _random_name()
    app = App.create(client, name=name)
    assert (app.error is None)
    assert (app.data is not None)
    app = app.data

    zip_bytes = create_app_zip(py_name)
    version = AppVersion.create(
        client,
        appId=app.id,
        filebytes=zip_bytes,
        configTemplate=versionConfigTemplate
    )
    # TODO: This is due to having to wait for the lambda to finish deploying.
    # TODO: We should update the task system to allow its .wait() to depend on this.
    time.sleep(15)
    version.wait()
    assert (version.error is None)
    assert (version.data is not None)
    version = version.data

    instance = AppInstance.create(
        client,
        appId=app.id,
        appVersionId=version.id,
        config=instanceConfig
    )
    instance.wait()
    assert (instance.error is None)
    assert (instance.data is not None)
    instance = instance.data

    assert (instance.appId == app.id)
    assert (instance.appVersionId == version.id)

    user = User.current(client).data

    assert (instance.userHandle == user.handle)
    assert (instance.userId == user.id)

    yield (app, version, instance)

    res = instance.delete()
    assert (res.error is None)

    res = version.delete()
    assert (res.error is None)

    res = app.delete()
    assert (res.error is None)


@contextlib.contextmanager
def deploy_plugin(py_name: str, plugin_type: str, versionConfigTemplate : Dict[str, any] = None, instanceConfig : Dict[str, any] = None, isTrainable: bool = False):
    client = _steamship()
    name = _random_name()
    plugin = Plugin.create(client, isTrainable=isTrainable, name=name, description='test', type=plugin_type, transport="jsonOverHttp",
                           isPublic=False)
    assert (plugin.error is None)
    assert (plugin.data is not None)
    plugin = plugin.data

    zip_bytes = create_app_zip(py_name)
    version = PluginVersion.create(
        client,
        "test-version",
        pluginId=plugin.id,
        filebytes=zip_bytes,
        configTemplate=versionConfigTemplate
    )
    # TODO: This is due to having to wait for the lambda to finish deploying.
    # TODO: We should update the task system to allow its .wait() to depend on this.
    time.sleep(15)
    version.wait()
    assert (version.error is None)
    assert (version.data is not None)
    version = version.data

    instance = PluginInstance.create(
        client,
        pluginId=plugin.id,
        pluginVersionId=version.id,
        config=instanceConfig
    )
    instance.wait()
    assert (instance.error is None)
    assert (instance.data is not None)
    instance = instance.data

    assert (instance.pluginId == plugin.id)
    assert (instance.pluginVersionId == version.id)

    user = User.current(client).data

    assert (instance.userId == user.id)

    yield (plugin, version, instance)

    res = instance.delete()
    assert (res.error is None)

    res = version.delete()
    assert (res.error is None)

    res = plugin.delete()
    assert (res.error is None)


def shouldUseSubdomain(client: Client):
    # We have to do a little switcheroo here depending on if we're hitting localhost or prod/staging.
    if 'localhost' in client.config.appBase or '127.0.0.1' in client.config.appBase:
        return False
    return True



from steamship.app import App, post, create_handler
from steamship.plugin.file_importer import FileImporter
from steamship.plugin.service import PluginResponse, PluginRequest
from steamship.extension.file import File

# Note: this aligns with the same document in the internal Engine test.
NAME = "Test Importer (Plugin)"
HANDLE = "test-importer-plugin-v1"
URL = "builtin://importer/test-plugin/v1"
TEST_H1 = "A Poem"
TEST_S1 = "Roses are red."
TEST_S2 = "Violets are blue."
TEST_S3 = "Sugar is sweet, and I love you."
TEST_DOC = "# {}\n\n{} {}\n\n{}\n".format(TEST_H1, TEST_S1, TEST_S2, TEST_S3)


class TestFileImporterPlugin(FileImporter, App):
    def run(self, request: PluginRequest[File.CreateRequest]) -> PluginResponse[File.CreateResponse]:
        return PluginResponse(
            data=File.CreateResponse(
                string=TEST_DOC
            )
        )

    @post('import')
    def do_import(self, **kwargs) -> any:
        importRequest = FileImporter.parse_request(request=kwargs)
        importResponse = self.run(importRequest)
        return FileImporter.response_to_dict(importResponse)


handler = create_handler(TestFileImporterPlugin)



from steamship import PluginInstance
from steamship.base.response import TaskStatus
from steamship.data.embeddings import IndexSnapshotRequest, IndexSnapshotResponse

from tests.client.helpers import _random_name, _steamship

__copyright__ = "Steamship"
__license__ = "MIT"

_TEST_EMBEDDER = "test-embedder"


def _insert(index, items):
    for item in items:
        index.insert(item, "TestId", "TestType", [1, 2, 3])

    # Now embed
    task = index.embed()
    task.wait()
    task.check()
    assert (task.task.state == TaskStatus.succeeded)


def _snapshot(index, windowSize=None):
    if windowSize is None:
        task = index.create_snapshot()
        task.wait()
        task.check()
    else:
        # We do this manually so as not to clutter the end-user visible
        # API with debug/testing parameters
        req = IndexSnapshotRequest(
            index.id,
            windowSize=windowSize
        )
        task = index.client.post(
            'embedding-index/snapshot/create',
            req,
            expect=IndexSnapshotResponse,
            asynchronous=True
        )
        task.wait()
        task.check()


def test_snapshot_create():
    steamship = _steamship()

    name = _random_name()
    pluginInstance = PluginInstance.create(steamship, pluginHandle=_TEST_EMBEDDER).data
    index = steamship.create_index(
        name=name,
        pluginInstance=pluginInstance.handle,
        upsert=True
    ).data

    _insert(index, ["Oranges are orange."])
    search_results = index.search("What color are oranges?", includeMetadata=True)
    assert (len(search_results.data.items) == 1)
    assert (search_results.data.items[0].value.indexSource == "index")
    assert (search_results.data.items[0].value.value == "Oranges are orange.")
    assert (search_results.data.items[0].value.externalId == "TestId")
    assert (search_results.data.items[0].value.externalType == "TestType")
    assert (len(search_results.data.items[0].value.metadata) == 3)

    _snapshot(index)
    search_results = index.search("What color are oranges?", includeMetadata=True)
    assert (len(search_results.data.items) == 1)
    assert (search_results.data.items[0].value.indexSource == "snapshot")
    assert (search_results.data.items[0].value.value == "Oranges are orange.")
    assert (search_results.data.items[0].value.externalId == "TestId")
    assert (search_results.data.items[0].value.externalType == "TestType")
    assert (len(search_results.data.items[0].value.metadata) == 3)

    _insert(index, ["Apples are red."])
    search_results = index.search("What color are apples?", includeMetadata=True)
    assert (len(search_results.data.items) == 1)
    assert (search_results.data.items[0].value.indexSource == "index")
    assert (search_results.data.items[0].value.value == "Apples are red.")
    assert (search_results.data.items[0].value.externalId == "TestId")
    assert (search_results.data.items[0].value.externalType == "TestType")
    assert (len(search_results.data.items[0].value.metadata) == 3)

    _snapshot(index)
    search_results = index.search("What color are apples?", includeMetadata=True)
    assert (len(search_results.data.items) == 1)
    assert (search_results.data.items[0].value.indexSource == "snapshot")
    assert (search_results.data.items[0].value.value == "Apples are red.")
    assert (search_results.data.items[0].value.externalId == "TestId")
    assert (search_results.data.items[0].value.externalType == "TestType")
    assert (len(search_results.data.items[0].value.metadata) == 3)

    index.delete()
    steamship = _steamship()

    name = _random_name()
    index = steamship.create_index(
        name=name,
        pluginInstance=pluginInstance.handle,
        upsert=True
    ).data

    sentences = []
    for i in range(15):
        sentences.append("Orange number {} is as good as the last".format(i))

    SENT = "Is orange number 13 any good?"
    _insert(index, sentences)

    search_results = index.search(SENT, includeMetadata=True)
    assert (len(search_results.data.items) == 1)
    assert (search_results.data.items[0].value.indexSource == "index")
    assert (search_results.data.items[0].value.value == "Orange number 13 is as good as the last")
    assert (search_results.data.items[0].value.externalId == "TestId")
    assert (search_results.data.items[0].value.externalType == "TestType")
    assert (len(search_results.data.items[0].value.metadata) == 3)

    _snapshot(index, windowSize=2)
    search_results = index.search(SENT, includeMetadata=True)
    assert (len(search_results.data.items) == 1)
    assert (search_results.data.items[0].value.indexSource == "snapshot")
    assert (search_results.data.items[0].value.value == "Orange number 13 is as good as the last")
    assert (search_results.data.items[0].value.externalId == "TestId")
    assert (search_results.data.items[0].value.externalType == "TestType")
    assert (len(search_results.data.items[0].value.metadata) == 3)

    index.delete()



from steamship import App

from tests.client.helpers import _random_name, _steamship

__copyright__ = "Steamship"
__license__ = "MIT"


def test_app_create():
    client = _steamship()
    name = _random_name()

    app = App.create(client, name=name)
    assert (app.error is None)
    assert (app.data is not None)
    assert (app.data.name == name)

    res = app.data.delete()
    assert (app.error is None)



import base64
import logging
from dataclasses import dataclass
from typing import Any

from steamship.base import Client
from steamship.base.mime_types import TEXT_MIME_TYPES


@dataclass
class RawDataPluginInput:
    pluginInstance: str = None
    data: Any = None
    defaultMimeType: str = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "RawDataPluginInput":
        if d is None:
            return None
        data = d.get('data', None)
        if data is not None and d.get('isBase64', False):
            data_bytes = base64.b64decode(data)
            if d.get('defaultMimeType', None) in TEXT_MIME_TYPES:
                data = data_bytes.decode('utf-8')
            else:
                data = data_bytes

        return RawDataPluginInput(
            pluginInstance=d.get('pluginInstance', None),
            data=data,
            defaultMimeType=d.get('defaultMimeType', None)
        )



from steamship.data.user import User

from .helpers import _steamship

__copyright__ = "Steamship"
__license__ = "MIT"


def test_connect():
    """Test basic connection"""
    client = _steamship()
    assert (client.config is not None)
    assert (client.config.profile == "test")
    assert (client.config.apiKey is not None)


def test_user():
    client = _steamship()
    user = User.current(client).data
    assert (user.id is not None)
    assert (user.handle is not None)



from steamship.plugin.inputs.block_and_tag_plugin_input import BlockAndTagPluginInput
from steamship.plugin.outputs.block_and_tag_plugin_output import BlockAndTagPluginOutput
from steamship.plugin.service import PluginRequest, PluginResponse
from steamship.extension.file import File
from steamship.data.block import Block

__copyright__ = "Steamship"
__license__ = "MIT"

from ..demo_apps.plugin_parser import TestParserPlugin

TEST_REQ = BlockAndTagPluginInput(
    file=File(
        blocks=[
            Block(id='ABC', text='Once upon a time there was a magical ship. The ship was powered by STEAM. The ship went to the moon.' )
        ]
    )
)
TEST_PLUGIN_REQ = PluginRequest(data=TEST_REQ)
TEST_REQ_DICT = TEST_PLUGIN_REQ.to_dict()


def _test_resp(res):
    assert (type(res) == PluginResponse)
    assert (type(res.data) == BlockAndTagPluginOutput)
    assert (len(res.data.file.blocks) == 1)
    assert (res.data.file.blocks[0].text == TEST_REQ.file.blocks[0].text)
    assert (len(res.data.file.blocks[0].tags) == 3)


def test_parser():
    parser = TestParserPlugin()
    res = parser.run(TEST_PLUGIN_REQ)
    _test_resp(res)



from dataclasses import dataclass

from steamship.base import Client


@dataclass
class ConvertRequest:
    type: str = None
    pluginInstance: str = None
    id: str = None
    handle: str = None
    name: str = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "ConvertRequest":
        return ConvertRequest(
            type=d.get('type', None),
            pluginInstance=d.get('pluginInstance', None),
            id=d.get('id', None),
            handle=d.get('handle', None),
            name=d.get('name', None)
        )



from typing import List

from steamship import Tag, TagKind, TextTag, Block, File
from steamship.app import App, post, create_handler, Response
from steamship.plugin.embedder import Embedder
from steamship.plugin.inputs.block_and_tag_plugin_input import BlockAndTagPluginInput
from steamship.plugin.outputs.block_and_tag_plugin_output import BlockAndTagPluginOutput
from steamship.plugin.service import PluginResponse, PluginRequest

FEATURES = ["employee", "roses", "run", "bike", "ted", "grace", "violets", "sugar", "sweet", "cake",
            "flour", "chocolate", "vanilla", "flavors", "flavor", "can", "armadillo", "pizza",
            "ship", "run", "jerry", "ronaldo", "ted", "brenda", "susan", "sam", "jerry", "shoe",
            "cheese", "water", "drink", "glass", "egg", "sleep", "code", "jonathan", "dolphin", "apple",
            "orange", "number", "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15",
            "16", "17", "bad", "terrible", "average", "good"]

DIMENSIONALITY = len(FEATURES)


def embed(s: str) -> List[float]:
    s = s.lower()
    return list(map(lambda word: 1.0 if word in s else 0.0, FEATURES))

def embedToTag(s: str) -> Tag.CreateRequest:
    embedding = embed(s)
    return Tag.CreateRequest(
        kind = TagKind.text,
        name =  TextTag.embedding,
        value = { TextTag.embedding : embedding }
    )

def embedBlock(block: Block) -> Block.CreateRequest:
    return Block.CreateRequest(
        id=block.id,
        text=block.text,
        tags=[embedToTag(block.text)]
    )


class TestEmbedderPlugin(Embedder, App):
    def run(self, request: PluginRequest[BlockAndTagPluginInput]) -> PluginResponse[BlockAndTagPluginOutput]:
        updatedBlocks = [ embedBlock(block) for block in request.data.file.blocks ]
        return PluginResponse(data=BlockAndTagPluginOutput(file=File.CreateRequest(blocks = updatedBlocks)))

    @post('tag')
    def embed(self, **kwargs) -> Response:
        embedRequest = Embedder.parse_request(request=kwargs)
        objResponse = self.run(embedRequest)
        dictResponse = Embedder.response_to_dict(objResponse)
        response = Response(json=dictResponse)
        return response


handler = create_handler(TestEmbedderPlugin)



"""

Please see https://docs.steamship.com/ for information about building a Steamship App

"""

import logging
from functools import wraps
from typing import Dict

from steamship.app.request import Request, Verb
from steamship.app.response import Error
from steamship.client.client import Steamship


def makeRegisteringDecorator(foreignDecorator):
    """
        Returns a copy of foreignDecorator, which is identical in every
        way(*), except also appends a .decorator property to the callable it
        spits out.
    """

    def newDecorator(func):
        # Call to newDecorator(method)
        # Exactly like old decorator, but output keeps track of what decorated it
        R = foreignDecorator(func)  # apply foreignDecorator, like call to foreignDecorator(method) would have done
        R.decorator = newDecorator  # keep track of decorator
        # R.original = func         # might as well keep track of everything!
        return R

    newDecorator.__name__ = foreignDecorator.__name__
    newDecorator.__doc__ = foreignDecorator.__doc__
    newDecorator.__is_endpoint__ = True
    # (*)We can be somewhat "hygienic", but newDecorator still isn't signature-preserving, i.e. you will not be able to get a runtime list of parameters. For that, you need hackish libraries...but in this case, the only argument is func, so it's not a big issue

    return newDecorator


# https://stackoverflow.com/questions/2366713/can-a-decorator-of-an-instance-method-access-the-class
def endpoint(verb: str = None, path: str = None):
    """By using **kw we can tag the function with any parameters"""

    def decorator(function):
        # This is used in conjunction with the __init_subclass__ code!
        # Otherwise the __name__ won't be correct in maybeDecorated.__name__!
        @wraps(function)
        def wrap(self, *args, **kwargs):
            return function(self, *args, **kwargs)

        wrap.__path__ = path
        wrap.__verb__ = verb
        return wrap

    decorator = makeRegisteringDecorator(decorator)
    return decorator


def get(path: str, **kwargs):
    return endpoint(verb='GET', path=path, **kwargs)


def post(path: str, **kwargs):
    return endpoint(verb='POST', path=path, **kwargs)


class App:
    """An Steamship microservice.

  This base.py class:

    1. Provide a pre-authenticated instance of the Steamship client
    2. Provides a Lambda handler that routes to registered functions
    3. Provides useful methods connecting functions to the router.
  """

    def __init__(self, client: Steamship = None, config: Dict[str, any] = None):
        self.client = client
        self.config = config

    """Base class to expose instance methods"""

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        cls._method_mappings = {}

        for maybeDecorated in cls.__dict__.values():
            if hasattr(maybeDecorated, 'decorator'):
                decorator = getattr(maybeDecorated, 'decorator')
                if hasattr(decorator, '__is_endpoint__') and getattr(decorator, '__is_endpoint__') == True:
                    path = getattr(maybeDecorated, '__path__') if hasattr(maybeDecorated, '__path__') else None
                    verb = getattr(maybeDecorated, '__verb__') if hasattr(maybeDecorated, '__verb__') else None
                    cls._register_mapping(maybeDecorated.__name__, verb, path)

    @classmethod
    def _register_mapping(cls, name: str, verb: str = None, path: str = None):
        """Registering a mapping permits the method to be invoked via HTTP."""

        if verb is None:
            verb = "get"
        if path is None and name is not None:
            path = "/{}".format(name)

        if getattr(cls, "_method_mappings") is None:
            setattr(cls, "_method_mappings", {})
        if path is None or path == '':
            path = '/'
        elif path[0] != '/':
            path = '/{}'.format(path)

        verb = Verb.safely_from_str(verb)
        if verb not in cls._method_mappings:
            cls._method_mappings[verb] = {}
        cls._method_mappings[verb][path] = name
        logging.info("[{}] {} {} => {}".format(cls.__name__, verb, path, name))

    def __call__(self, request: Request, context: any = None):
        """Invokes a method call if it is registered."""
        if not getattr(self.__class__, "_method_mappings"):
            return Error(
                httpStatus=404,
                message="No mappings available for app."
            )

        if request.invocation is None:
            return Error(
                httpStatus=404,
                message="No invocation was found."
            )

        verb = Verb.safely_from_str(request.invocation.httpVerb)
        appPath = request.invocation.appPath
        arguments = request.invocation.arguments

        if appPath is None or appPath == '':
            appPath = '/'
        elif appPath[0] != '/':
            appPath = '/{}'.format(appPath)

        if verb not in self.__class__._method_mappings:
            return Error(
                httpStatus=404,
                message="No methods for verb {} available.".format(verb)
            )
        if appPath not in self.__class__._method_mappings[verb]:
            return Error(
                httpStatus=404,
                message="No handler for {} {} available.".format(verb, appPath)
            )
        method = self.__class__._method_mappings[verb][appPath]
        if not (hasattr(self, method) and callable(getattr(self, method))):
            return Error(
                httpStatus=500,
                message="Handler for {} {} not callable.".format(verb, appPath)
            )

        if arguments is None:
            return getattr(self, method)()
        else:
            return getattr(self, method)(**arguments)



from steamship.base import Client
from steamship.base.response import TaskStatus
from steamship.data.embeddings import EmbeddedItem
from steamship.data.plugin_instance import PluginInstance

from tests.client.helpers import _random_index, _random_name, _steamship

__copyright__ = "Steamship"
__license__ = "MIT"

_TEST_EMBEDDER = "test-embedder"


def create_index(client: Client, pluginInstance: str):
    steamship = _steamship()
    name = _random_name()

    # Should require plugin
    task = steamship.create_index(
        name="Test Index"
    )
    assert task.error is not None
    assert task.data is None

    index = steamship.create_index(
        name=name,
        pluginInstance=pluginInstance,
        upsert=True
    ).data
    assert index is not None

    # Duplicate creation should fail with upsert=False
    task = steamship.create_index(
        handle=index.handle,
        pluginInstance=pluginInstance,
        upsert=False
    )
    assert task.error is not None
    assert task.data is None

    index.delete()


def test_create_index():
    steamship = _steamship()
    pluginInstance = PluginInstance.create(steamship, pluginHandle=_TEST_EMBEDDER).data
    create_index(_steamship(), pluginInstance.handle)


def test_delete_index():
    steamship = _steamship()
    name = _random_name()
    pluginInstance = PluginInstance.create(steamship, pluginHandle=_TEST_EMBEDDER).data
    index = steamship.create_index(
        name=name,
        pluginInstance=pluginInstance.handle,
        upsert=True
    ).data
    assert (index.id is not None)

    task = steamship.create_index(
        handle=index.handle,
        pluginInstance=pluginInstance.handle,
        upsert=True
    )
    assert task.error is None
    index2 = task.data
    assert (index.id == index2.id)

    index.delete()

    task = steamship.create_index(
        name=name,
        pluginInstance=pluginInstance.handle,
        upsert=True
    )
    assert task.error is None
    assert task.data is not None
    index3 = task.data
    assert (index.id != index3.id)
    index3.delete()


def _list_equal(actual, expected):
    assert len(actual) == len(expected)
    assert all([a == b for a, b in zip(actual, expected)])


def test_insert_many():
    steamship = _steamship()
    name = _random_name()
    pluginInstance = PluginInstance.create(steamship, pluginHandle=_TEST_EMBEDDER).data
    with _random_index(steamship, pluginInstance.handle) as index:
        item1 = EmbeddedItem(
            value="Pizza",
            externalId="pizza",
            externalType="food",
            metadata=[1, 2, 3]
        )
        item2 = EmbeddedItem(
            value="Rocket Ship",
            externalId="space",
            externalType="vehicle",
            metadata="Foo"
        )

        index.insert_many([item1, item2])
        index.embed().wait()

        task = index.list_items()
        assert task.error is None
        indexItems = task.data
        assert (len(indexItems.items) == 2)
        assert (len(indexItems.items[0].embedding) > 0)
        assert (len(indexItems.items[1].embedding) > 0)
        assert (len(indexItems.items[0].embedding) == len(indexItems.items[1].embedding))

        res = index.search(item1.value, includeMetadata=True, k=100)
        assert (res.data.items is not None)
        assert (len(res.data.items) == 2)
        assert (res.data.items[0].value.value == item1.value)
        assert (res.data.items[0].value.externalId == item1.externalId)
        assert (res.data.items[0].value.externalType == item1.externalType)
        _list_equal(res.data.items[0].value.metadata, item1.metadata)

        res = index.search(item2.value, includeMetadata=True)
        assert (res.data.items is not None)
        assert (res.data.items[0].value.value == item2.value)
        assert (res.data.items[0].value.externalId == item2.externalId)
        assert (res.data.items[0].value.externalType == item2.externalType)
        assert (res.data.items[0].value.metadata == item2.metadata)


def test_embed_task():
    steamship = _steamship()
    name = _random_name()
    pluginInstance = PluginInstance.create(steamship, pluginHandle=_TEST_EMBEDDER).data
    with _random_index(steamship, pluginInstance.handle) as index:
        insert_results = index.insert("test", reindex=False)
        res = index.embed()

        assert (res.task.taskId is not None)
        assert (res.task.state is not None)
        assert (res.task.taskCreatedOn is not None)
        assert (res.task.taskLastModifiedOn is not None)
        assert (res.task.state == TaskStatus.waiting)
        res.wait()
        assert (res.task.state == TaskStatus.succeeded)


def test_duplicate_inserts():
    steamship = _steamship()
    name = _random_name()

    pluginInstance = PluginInstance.create(steamship, pluginHandle=_TEST_EMBEDDER).data
    with _random_index(steamship, pluginInstance.handle) as index:
        # Test for suppressed re-indexing
        A1 = "Ted can eat an entire block of cheese."
        Q1 = "Who can eat the most cheese"
        insert_results = index.insert(A1)
        search_results = index.search(Q1)


def test_index_usage():
    steamship = _steamship()
    name = _random_name()

    pluginInstance = PluginInstance.create(steamship, pluginHandle=_TEST_EMBEDDER).data
    with _random_index(steamship, pluginInstance.handle) as index:
        A1 = "Ted can eat an entire block of cheese."
        Q1 = "Who can eat the most cheese"
        insert_results = index.insert(A1)
        search_results = index.search(Q1)

        # Now embed
        task = index.embed()
        task.wait()
        task.check()
        assert (task.task.state == TaskStatus.succeeded)

        search_results = index.search(Q1)
        assert (len(search_results.data.items) == 1)
        assert (search_results.data.items[0].value.value == A1)

        # Associate metadata
        A2 = "Armadillo shells are bulletproof."
        Q2 = "What is something interesting about Armadillos?"
        A2id = "A2id"
        A2type = "A2type"
        A2metadata = dict(
            id=A2id,
            idid="{}{}".format(A2id, A2id),
            boolVal=True,
            intVal=123,
            floatVal=1.2
        )

        insert_results2 = index.insert(
            A2,
            externalId=A2id,
            externalType=A2type,
            metadata=A2metadata
        )
        search_results2 = index.search(Q2)
        assert (len(search_results2.data.items) == 1)
        assert (search_results2.data.items[0].value.value == A2)
        assert (search_results2.data.items[0].value.externalId == None)
        assert (search_results2.data.items[0].value.externalType == None)
        assert (search_results2.data.items[0].value.metadata == None)

        search_results3 = index.search(Q2, includeMetadata=True)
        assert (len(search_results3.data.items) == 1)
        assert (search_results3.data.items[0].value.value == A2)
        assert (search_results3.data.items[0].value.externalId == A2id)
        assert (search_results3.data.items[0].value.externalType == A2type)

        assert (search_results3.data.items[0].value.metadata == A2metadata)
        # Because I don't know pytest enough to fully trust the dict comparison..
        assert (search_results3.data.items[0].value.metadata["id"] == A2id)
        assert (search_results3.data.items[0].value.metadata["idid"] == "{}{}".format(A2id, A2id))

        search_results4 = index.search(Q2, k=10)
        assert (len(search_results4.data.items) == 2)
        assert (search_results4.data.items[0].value.value == A2)
        assert (search_results4.data.items[1].value.value == A1)


def test_multiple_queries():
    steamship = _steamship()
    name = _random_name()

    pluginInstance = PluginInstance.create(steamship, pluginHandle=_TEST_EMBEDDER).data
    with _random_index(steamship, pluginInstance.handle) as index:
        # Test for suppressed re-indexing
        A1 = "Ted can eat an entire block of cheese."
        A2 = "Joe can drink an entire glass of water."
        insert_results = index.insert_many([A1, A2])
        index.embed().wait()

        QS1 = ["Who can eat the most cheese", "Who can run the fastest?"]
        search_results = index.search(QS1)
        assert (len(search_results.data.items) == 1)
        assert (search_results.data.items[0].value.value == A1)
        assert (search_results.data.items[0].value.query == QS1[0])

        QS2 = ["Who can tie a shoe?", "Who can drink the most water?"]
        search_results = index.search(QS2)
        assert (len(search_results.data.items) == 1)
        assert (search_results.data.items[0].value.value == A2)
        assert (search_results.data.items[0].value.query == QS2[1])

        QS3 = ["What can Ted do?", "What can Sam do?", "What can Jerry do?"]
        search_results = index.search(QS3)
        assert (len(search_results.data.items) == 1)
        assert (search_results.data.items[0].value.value == A1)
        assert (search_results.data.items[0].value.query == QS3[0])

        QS3 = ["What can Sam do?", "What can Ted do?", "What can Jerry do?"]
        search_results = index.search(QS3)
        assert (len(search_results.data.items) == 1)
        assert (search_results.data.items[0].value.value == A1)
        assert (search_results.data.items[0].value.query == QS3[1])

        index.create_snapshot().wait()

        A3 = "Susan can run very fast."
        A4 = "Brenda can fight alligators."
        insert_results = index.insert_many([A3, A4])
        index.embed().wait()

        QS4 = ["What can Brenda do?", "What can Ronaldo do?", "What can Jerry do?"]
        search_results = index.search(QS4)
        assert (len(search_results.data.items) == 1)
        assert (search_results.data.items[0].value.value == A4)
        assert (search_results.data.items[0].value.query == QS4[0])

        QS4 = ["What can Brenda do?", "Who should run a marathon?", "What can Jerry do?"]
        search_results = index.search(QS4, k=2)
        assert (len(search_results.data.items) == 2)
        assert (search_results.data.items[0].value.value == A4)
        assert (search_results.data.items[0].value.query == QS4[0])
        assert (search_results.data.items[1].value.value == A3)
        assert (search_results.data.items[1].value.query == QS4[1])


def test_empty_queries():
    steamship = _steamship()
    name = _random_name()

    pluginInstance = PluginInstance.create(steamship, pluginHandle=_TEST_EMBEDDER).data
    with _random_index(steamship, pluginInstance.handle) as index:
        A1 = "Ted can eat an entire block of cheese."
        A2 = "Joe can drink an entire glass of water."
        insert_results = index.insert_many([A1, A2])
        index.embed().wait()

        search_results = index.search(None)
        assert (search_results.error is not None)
        assert (search_results.data is None)

        # These technically don't count as empty. Leaving this test in here
        # to encode and capture that in case we want to change it.
        search_results = index.search([])
        assert (len(search_results.data.items) == 0)

        search_results = index.search("")
        assert (len(search_results.data.items) == 1)



from steamship import Corpus
from steamship.plugin.corpus_importer import CorpusImportResponse, CorpusImportRequest


def doImport(
        self,
        value: str = None,
        url: str = None,
        pluginInstance: str = None,
        spaceId: str = None,
        spaceHandle: str = None,
        space: any = None,
        fileImporterPluginInstance: str = None
) -> "Response[CorpusImportResponse]":
    req = CorpusImportRequest(
        type="corpus",
        id=self.id,
        value=value,
        url=url,
        pluginInstance=pluginInstance,
        fileImporterPluginInstance=fileImporterPluginInstance
    )
    return self.client.post(
        'plugin/instance/importCorpus',
        req,
        expect=CorpusImportResponse,
        spaceId=spaceId,
        spaceHandle=spaceHandle,
        space=space
    )


Corpus.doImport = doImport



from abc import ABC

from steamship.base import Client
from steamship.plugin.inputs.file_import_plugin_input import FileImportPluginInput
from steamship.plugin.outputs.raw_data_plugin_output import RawDataPluginOutput
from steamship.plugin.service import PluginService, PluginRequest

# Note!
# =====
#
# This is the PLUGIN IMPLEMENTOR's View of a File Importer.
#
# If you are using the Steamship Client, you probably want steamship.client.operations.file_importer instead
# of this file.
#
class FileImporter(PluginService[FileImportPluginInput, RawDataPluginOutput], ABC):
    @classmethod
    def subclass_request_from_dict(cls, d: any, client: Client = None) -> PluginRequest[FileImportPluginInput]:
        return FileImportPluginInput.from_dict(d, client=client)



from steamship import File
from steamship.app import App, post, create_handler
from steamship.client.operations.corpus_importer import CorpusImportRequest, CorpusImportResponse
from steamship.plugin.corpus_importer import CorpusImporter
from steamship.plugin.service import PluginResponse, PluginRequest


class TestCorpusImporterPlugin(CorpusImporter, App):
    def run(self, request: PluginRequest[CorpusImportRequest]) -> PluginResponse[CorpusImportResponse]:
        return PluginResponse(
            data=CorpusImportResponse(
                fileImportRequests=[
                    File.CreateRequest(
                        type="fileImporter",
                        pluginInstance=request.data.fileImporterPluginInstance
                        # This is a test importer built into the Steamship Engine
                    ),
                    File.CreateRequest(
                        type="fileImporter",
                        pluginInstance=request.data.fileImporterPluginInstance
                    )
                ]
            )
        )

    @post('import')
    def do_import(self, **kwargs) -> any:
        request = CorpusImporter.parse_request(request=kwargs)
        response = self.run(request)
        return CorpusImporter.response_to_dict(response)


handler = create_handler(TestCorpusImporterPlugin)



import requests

from steamship.data.user import User
from tests.client.helpers import _steamship, deploy_app, shouldUseSubdomain

__copyright__ = "Steamship"
__license__ = "MIT"


def test_instance_invoke():
    with deploy_app("demo_app.py") as (app, version, instance):
        # Now let's invoke it!
        # Note: we're invoking the data at demo_app.py in the tests/demo_apps folder
        client = _steamship()
        res = instance.get('greet').data
        assert (res == "Hello, Person!")

        useSubdomain = shouldUseSubdomain(client)

        # Also try with raw http
        user = User.current(client).data
        url = instance.full_url_for("greet")
        resp = requests.get(url, headers=dict(authorization="Bearer {}".format(client.config.apiKey)))
        assert (resp.text == "Hello, Person!")

        res = instance.get('greet', name="Ted").data
        assert (res == "Hello, Ted!")
        url = instance.full_url_for("greet?name=Ted")
        resp = requests.get(url, headers=dict(authorization="Bearer {}".format(client.config.apiKey)))
        assert (resp.text == "Hello, Ted!")

        res = instance.post('greet').data
        assert (res == "Hello, Person!")
        url = instance.full_url_for("greet")
        resp = requests.post(url, headers=dict(authorization="Bearer {}".format(client.config.apiKey)))
        assert (resp.text == "Hello, Person!")

        res = instance.post('greet', name="Ted").data
        assert (res == "Hello, Ted!")
        url = instance.full_url_for("greet")
        resp = requests.post(url, json=dict(name="Ted"),
                             headers=dict(authorization="Bearer {}".format(client.config.apiKey)))
        assert (resp.text == "Hello, Ted!")



import json

from steamship import PluginInstance
from ..client.helpers import deploy_plugin, _steamship
from tests.client.operations.test_tag_file import parse_file

__copyright__ = "Steamship"
__license__ = "MIT"


def test_e2e_parser():
    client = _steamship()

    configTemplate = {"tagKind": {"type":"string"},
                    "tagName": {"type":"string"},
                    "numberValue": {"type":"number"},
                    "booleanValue": {"type":"boolean"}}
    instanceConfig1 = {"tagKind": "testTagKind",
                         "tagName": "testTagName",
                        "numberValue": 3,
                        "booleanValue": True}

    with deploy_plugin("plugin_configurable_tagger.py", "tagger", versionConfigTemplate=configTemplate, instanceConfig=instanceConfig1) as (plugin, version, instance):
        test_doc = "Hi there"
        res = client.tag(doc=test_doc, pluginInstance=instance.handle)
        res.wait()
        assert (res.error is None)
        assert (res.data is not None)
        assert (len(res.data.file.blocks) == 1)
        assert (res.data.file.blocks[0].text == test_doc)

        #Validate configured content
        assert (len(res.data.file.tags) == 1)
        tag = res.data.file.tags[0]
        assert (tag.name == instanceConfig1['tagName'])
        assert (tag.kind == instanceConfig1['tagKind'])
        tagValue = json.loads(tag.value)
        assert (tagValue['numberValue'] == instanceConfig1['numberValue'])
        assert (tagValue['booleanValue'] == instanceConfig1['booleanValue'])

        instanceConfig2 = {"tagKind": "testTagKind2",
                           "tagName": "testTagName2",
                           "numberValue": 4,
                           "booleanValue": False}

        instance2 = PluginInstance.create(
            client,
            pluginId=plugin.id,
            pluginVersionId=version.id,
            config=instanceConfig2
        )
        instance2.wait()
        assert (instance2.error is None)
        assert (instance2.data is not None)
        instance2 = instance2.data

        res = client.tag(doc=test_doc, pluginInstance=instance2.handle)
        res.wait()
        assert (res.error is None)
        assert (res.data is not None)
        assert (len(res.data.file.blocks) == 1)
        assert (res.data.file.blocks[0].text == test_doc)

        # Validate configured content
        assert (len(res.data.file.tags) == 1)
        tag = res.data.file.tags[0]
        assert (tag.name == instanceConfig2['tagName'])
        assert (tag.kind == instanceConfig2['tagKind'])
        tagValue = json.loads(tag.value)
        assert (tagValue['numberValue'] == instanceConfig2['numberValue'])
        assert (tagValue['booleanValue'] == instanceConfig2['booleanValue'])









from dataclasses import dataclass
from typing import List, Dict

from steamship.base import Client, Request
from steamship.base.response import Response


class PluginVersion:
    pass


@dataclass
class CreatePluginVersionRequest(Request):
    pluginId: str = None
    name: str = None
    handle: str = None
    upsert: bool = None
    type: str = 'file'
    configTemplate: Dict[str, any] = None


@dataclass
class DeletePluginVersionRequest(Request):
    id: str


@dataclass
class ListPublicPluginVersionsRequest(Request):
    handle: str
    pluginId: str


@dataclass
class ListPrivatePluginVersionsRequest(Request):
    handle: str
    pluginId: str


@dataclass
class ListPluginVersionsResponse(Request):
    plugins: List[PluginVersion]

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "ListPluginsResponse":
        return ListPluginVersionsResponse(
            plugins=[PluginVersion.from_dict(x, client=client) for x in (d.get("pluginVersions", []) or [])]
        )


@dataclass
class PluginVersion:
    client: Client = None
    id: str = None
    pluginId: str = None
    name: str = None
    handle: str = None
    configTemplate: Dict[str, any] = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "PluginVersion":
        if 'pluginVersion' in d:
            d = d['pluginVersion']

        return PluginVersion(
            client=client,
            id=d.get('id', None),
            name=d.get('name', None),
            handle=d.get('handle', None),
            configTemplate=d.get('configTemplate', None)
        )

    @staticmethod
    def create(
            client: Client,
            handle: str,
            pluginId: str = None,
            name: str = None,
            filename: str = None,
            filebytes: bytes = None,
            upsert: bool = None,
            configTemplate: Dict[str, any] = None
    ) -> Response[PluginVersion]:

        if filename is None and filebytes is None:
            raise Exception("Either filename or filebytes must be provided.")
        if filename is not None and filebytes is not None:
            raise Exception("Only either filename or filebytes should be provided.")

        if filename is not None:
            with open(filename, 'rb') as f:
                filebytes = f.read()

        req = CreatePluginVersionRequest(
            name=name,
            handle=handle,
            pluginId=pluginId,
            upsert=upsert,
            configTemplate=configTemplate
        )

        return client.post(
            'plugin/version/create',
            payload=req,
            file=('plugin.zip', filebytes, "multipart/form-data"),
            expect=PluginVersion
        )

    def delete(self) -> "PluginVersion":
        req = DeletePluginVersionRequest(
            id=self.id
        )
        return self.client.post(
            'plugin/version/delete',
            payload=req,
            expect=PluginVersion
        )

    @staticmethod
    def getPublic(client: Client, pluginId: str, handle: str):
        publicPlugins = PluginVersion.listPublic(client=client, pluginId=pluginId, handle=handle).data.plugins
        for plugin in publicPlugins:
            if plugin.handle == handle:
                return plugin
        return None

    @staticmethod
    def listPublic(
            client: Client,
            pluginId: str = None,
            handle: str = None
    ) -> Response[ListPluginVersionsResponse]:
        return client.post(
            'plugin/version/public',
            ListPublicPluginVersionsRequest(handle=handle, pluginId=pluginId),
            expect=ListPluginVersionsResponse,
        )

    @staticmethod
    def listPrivate(
            client: Client,
            type=None,
            pluginId: str = None,
            handle: str = None
    ) -> Response[ListPluginVersionsResponse]:
        return client.post(
            'plugin/version/private',
            ListPrivatePluginVersionsRequest(handle=handle, pluginId=pluginId),
            expect=ListPluginVersionsResponse,
        )



import logging

from steamship.base import Response, TaskComment, Client
from steamship.base.tasks import TaskCommentList

__copyright__ = "Steamship"
__license__ = "MIT"

_logger = logging.getLogger(__name__)


class Tasks:
    """Asynchronous background task (and task feedback) management.
    """

    def __init__(self, client: Client):
        self.client = client

    def list_comments(
            self,
            taskId: str = None,
            externalId: str = None,
            externalType: str = None,
            externalGroup: str = None
    ) -> Response[TaskCommentList]:
        return TaskComment.list(
            client=self.client,
            taskId=taskId,
            externalId=externalId,
            externalType=externalType,
            externalGroup=externalGroup
        )



from steamship import MimeTypes, DocTag
from steamship.base.response import TaskStatus
from tests.client.helpers import _random_name, _steamship

__copyright__ = "Steamship"
__license__ = "MIT"


def test_file_upload_then_parse():
    steamship = _steamship()

    name_a = "{}.txt".format(_random_name())
    a = steamship.upload(
        name=name_a,
        content="This is a test."
    ).data
    assert (a.id is not None)

    q1 = a.query().data
    assert (len(q1.blocks) == 0)

    task = a.convert(pluginInstance="markdown-converter-default-1.0")
    assert (task.error is None)
    assert (task.task is not None)
    assert (task.task.state == TaskStatus.waiting)

    task.wait()
    assert (task.error is None)
    assert (task.task is not None)
    assert (task.task.state == TaskStatus.succeeded)

    q1 = a.query().data
    assert (len(q1.blocks) == 1)
    assert (q1.blocks[0].text == 'This is a test.')

    name_b = "{}.mkd".format(_random_name())
    b = steamship.upload(
        name=name_b,
        content="""# Header

This is a test."""
    ).data
    assert (b.id is not None)

    q1 = b.query().data
    assert (len(q1.blocks) == 0)

    task = b.convert(pluginInstance="markdown-converter-default-1.0")
    assert (task.error is None)
    assert (task.task is not None)
    task.wait()

    q1 = b.query().data
    assert (len(q1.blocks) == 2)
    assert (q1.blocks[1].text == 'This is a test.')
    assert (q1.blocks[0].text == 'Header')

    a.delete()
    b.delete()





import base64
import dataclasses
import io
import json as jsonlib
from typing import Any, Tuple

from steamship.base.mime_types import MimeTypes


def guess_mime(obj: Any, provided_mime: str = None) -> str:
    if provided_mime is not None:
        return provided_mime
    if type(obj) in [str, int, float, bool]:
        return MimeTypes.TXT
    return MimeTypes.BINARY


def flexi_create(
        body: any = None,
        string: str = None,
        json: Any = None,
        bytes: io.BytesIO = None,
        mimeType=None) -> Tuple[Any, str]:
    """
    It's convenient for some constructors to accept a variety of input types:
    - body (your chocie)
    - string
    - json
    - bytes

    .. And have them all homogenized.
    """
    if body is not None:
        return body, guess_mime(body, mimeType)

    if string is not None:
        return string, mimeType or MimeTypes.TXT

    if json is not None:
        if dataclasses.is_dataclass(json):
            return jsonlib.dumps(dataclasses.asdict(json)), mimeType or MimeTypes.JSON
        return jsonlib.dumps(json), mimeType or MimeTypes.JSON

    if bytes is not None:
        return base64.b64encode(bytes).decode('utf-8'), mimeType or MimeTypes.BINARY

    return None, None



import json
from dataclasses import dataclass

from steamship.base import Client


@dataclass
class Hit:
    id: str = None
    index: int = None
    indexSource: str = None
    value: str = None
    score: float = None
    externalId: str = None
    externalType: str = None
    metadata: any = None
    query: str = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "Hit":
        metadata = d.get("metadata", None)
        if metadata is not None:
            try:
                metadata = json.loads(metadata)
            except:
                pass

        return Hit(
            id=d.get("id", None),
            index=d.get("index", None),
            indexSource=d.get("indexSource", None),
            value=d.get("value", d.get("text", None)),
            score=d.get("score", None),
            externalId=d.get("externalId", None),
            externalType=d.get("externalType", None),
            metadata=metadata,
            query=d.get("query", None)
        )



from steamship import PluginInstance, File
from steamship.base import Client

from tests.client.helpers import _steamship

__copyright__ = "Steamship"
__license__ = "MIT"

_TEST_EMBEDDER = "test-embedder"

def count_embeddings(file: File):
    embeddings = 0
    for block in file.blocks:
        for tag in block.tags:
            if tag.kind == 'text' and tag.name == 'embedding':
                embeddings += 1
    return embeddings

def basic_embeddings(steamship: Client, pluginInstance: str):
    e1 = steamship.tag("This is a test", pluginInstance=pluginInstance)
    e1b = steamship.tag("Banana", pluginInstance=pluginInstance)
    e1.wait()
    e1b.wait()
    assert (count_embeddings(e1.data.file) == 1)
    assert (count_embeddings(e1b.data.file) == 1)
    assert (len(e1.data.file.blocks[0].tags[0].value['embedding']) > 1)

    e2 = steamship.tag("This is a test", pluginInstance=pluginInstance)
    e2.wait()
    assert (count_embeddings(e2.data.file) == 1)
    assert (len(e2.data.file.blocks[0].tags[0].value['embedding']) == len(e1.data.file.blocks[0].tags[0].value['embedding']))

    e4 = steamship.tag("This is a test", pluginInstance=pluginInstance)
    e4.wait()
    assert (count_embeddings(e4.data.file) == 1)


def test_basic_embeddings():
    pluginInstance = PluginInstance.create(_steamship(), pluginHandle=_TEST_EMBEDDER).data
    basic_embeddings(_steamship(), pluginInstance.handle)


def basic_embedding_search(steamship: Client, pluginInstance: str):
    docs = [
        "Armadillo shells are bulletproof.",
        "Dolphins sleep with one eye open.",
        "Alfred Hitchcock was frightened of eggs.",
        "Jonathan can help you with new employee onboarding",
        "The code for the New York office is 1234",
    ]
    query = "Who should I talk to about new employee setup?"
    results = steamship.embed_and_search(query, docs, pluginInstance=pluginInstance)
    assert (len(results.data.items) == 1)
    assert (results.data.items[0].value.value == "Jonathan can help you with new employee onboarding")


def test_basic_embedding_search():
    pluginInstance = PluginInstance.create(_steamship(), pluginHandle=_TEST_EMBEDDER).data
    basic_embedding_search(_steamship(), pluginInstance.handle)



from dataclasses import dataclass
from typing import Dict

from steamship.base import Client


@dataclass
class FileImportPluginInput:
    value: str = None
    data: str = None
    url: str = None
    pluginInstance: str = None
    mimeType: str = None

    @staticmethod
    def from_dict(d: any = None, client: Client = None) -> "FileImportPluginInput":
        if d is None:
            return None

        return FileImportPluginInput(
            value = d.get('value', None),
            data = d.get('data', None),
            url = d.get('url', None),
            pluginInstance = d.get('pluginInstance', None),
            mimeType = d.get('mimeType', None),
        )

    def to_dict(self) -> Dict:
        if self.file is None:
            return dict()
        return dict(file=self.file.to_dict())



from abc import ABC

from steamship.base import Client
from steamship.plugin.inputs.raw_data_plugin_input import RawDataPluginInput
from steamship.plugin.outputs.block_and_tag_plugin_output import BlockAndTagPluginOutput
from steamship.plugin.service import PluginService, PluginRequest

# Note!
# =====
#
# This is the PLUGIN IMPLEMENTOR's View of a Converter.
#
# If you are using the Steamship Client, you probably want steamship.client.operations.converter instead
# of this file.
#
class Converter(PluginService[RawDataPluginInput, BlockAndTagPluginOutput], ABC):
    @classmethod
    def subclass_request_from_dict(cls, d: any, client: Client = None) -> PluginRequest[RawDataPluginInput]:
        return RawDataPluginInput.from_dict(d, client=client)




from dataclasses import dataclass
from typing import Dict

from steamship.base import Client, Request


@dataclass
class CreateAppInstanceRequest(Request):
    id: str = None
    appId: str = None
    appVersionId: str = None
    name: str = None
    handle: str = None
    upsert: bool = None
    config: Dict[str, any] = None


@dataclass
class DeleteAppInstanceRequest(Request):
    id: str


@dataclass
class AppInstance:
    client: Client = None
    id: str = None
    name: str = None
    handle: str = None
    appId: str = None
    appHandle: str = None
    userHandle: str = None
    appVersionId: str = None
    userId: str = None
    invocationURL: str = None
    config: Dict[str, any] = None
    spaceId: str = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "AppInstance":
        if 'appInstance' in d:
            d = d['appInstance']

        return AppInstance(
            client=client,
            id=d.get('id', None),
            name=d.get('name', None),
            handle=d.get('handle', None),
            appId=d.get('appId', None),
            appHandle=d.get('appHandle', None),
            userHandle=d.get('userHandle', None),
            appVersionId=d.get('appVersionId', None),
            userId=d.get('userId', None),
            invocationURL=d.get('invocationURL', None),
            config= d.get('config', None),
            spaceId= d.get('spaceId', None)
        )

    @staticmethod
    def create(
            client: Client,
            appId: str = None,
            appVersionId: str = None,
            name: str = None,
            handle: str = None,
            upsert: bool = None,
            config: Dict[str, any] = None
    ) -> "AppInstance":

        req = CreateAppInstanceRequest(
            name=name,
            handle=handle,
            appId=appId,
            appVersionId=appVersionId,
            upsert=upsert,
            config=config
        )

        return client.post(
            'app/instance/create',
            payload=req,
            expect=AppInstance
        )

    def delete(self) -> "AppInstance":
        req = DeleteAppInstanceRequest(
            id=self.id
        )
        return self.client.post(
            'app/instance/delete',
            payload=req,
            expect=AppInstance
        )

    def get(self, path: str, **kwargs):
        if path[0] == '/':
            path = path[1:]
        return self.client.get(
            '/_/_/{}'.format(path),
            payload=kwargs,
            appCall=True,
            appOwner=self.userHandle,
            appId=self.appId,
            appInstanceId=self.id,
            spaceId=self.spaceId
        )

    def post(self, path: str, **kwargs):
        if path[0] == '/':
            path = path[1:]
        return self.client.post(
            '/_/_/{}'.format(path),
            payload=kwargs,
            appCall=True,
            appOwner=self.userHandle,
            appId=self.appId,
            appInstanceId=self.id,
            spaceId=self.spaceId
        )

    def full_url_for(self, path: str):
        return "{}{}".format(
            self.invocationURL,
            path
        )


@dataclass
class ListPrivateAppInstancesRequest(Request):
    pass



from dataclasses import dataclass
from typing import List
from steamship.base import Client

@dataclass
class EmbeddedItemsPluginOutput:
    embeddings: List[List[float]]

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "EmbeddedItemPluginOutput":
        return EmbeddedItemsPluginOutput(
            embeddings=d.get('embeddings', None),
        )

    def to_dict(self) -> dict:
        return dict(
            embeddings=self.embeddings
        )



from steamship import Steamship
from steamship.app import get, App, Response, Error, post, create_handler

class TestApp(App):

    @get('greet')
    def greet1(self, name: str = "Person") -> Response:
        return Response(string='Hello, {}!'.format(name))

    @post('greet')
    def greet2(self, name: str = "Person") -> Response:
        return Response(string='Hello, {}!'.format(name))

    @get('space')
    def space(self) -> Response:
        return Response(string=self.client.config.spaceId)

    @get('config')
    def config(self) -> Response:
        return Response(json=dict(
            spaceId=self.client.config.spaceId,
            appBase=self.client.config.appBase,
            Client=self.client.config.apiBase,
            apiKey=self.client.config.apiKey
        ))

    @post('learn')
    def learn(self, fact: str = None) -> Response:
        """Learns a new fact."""
        if fact is None:
            return Error(message="Empty fact provided to learn.")

        if self.index is None:
            return Error(message="Unable to initialize QA index.")

        res = self.index.embed(fact)

        if res.error:
            # Steamship error messages can be passed straight
            # back to the user
            return Error(
                message=res.error.statusMessage,
                suggestion=res.error.statusSuggestion,
                code=res.error.statusCode
            )

        return Response(json=res.data)

    @post('query')
    def query(self, query: str = None, k: int = 1) -> Response:
        """Learns a new fact."""
        if query is None:
            return Error(message="Empty query provided.")

        if self.index is None:
            return Error(message="Unable to initialize QA index.")

        res = self.index.query(query=query, k=k)

        if res.error:
            # Steamship error messages can be passed straight
            # back to the user
            return Error(
                message=res.error.statusMessage,
                suggestion=res.error.statusSuggestion,
                code=res.error.statusCode
            )

        return Response(json=res.data)


handler = create_handler(TestApp)



import pytest

from steamship import MimeTypes, File, Corpus
from tests.client.helpers import _random_name, _steamship

__copyright__ = "Steamship"
__license__ = "MIT"


def test_corpus_create():
    client = _steamship()

    # Should require name
    corpus = Corpus.create(client)
    assert (corpus.data is not None)
    corpus.data.delete()

    corpus = Corpus.create(client, name=_random_name()).data

    name_a = "{}.mkd".format(_random_name())
    a = File.create(
        client,
        name=name_a,
        content="A",
        corpusId=corpus.id,
        mimeType=MimeTypes.MKD
    ).data
    assert (a.id is not None)
    assert (a.name == name_a)
    assert (a.mimeType == MimeTypes.MKD)
    assert (a.corpusId == corpus.id)

    name_a = "{}.html".format(_random_name())
    a = File.scrape(
        client,
        name=name_a,
        corpusId=corpus.id,
        url="https://edwardbenson.com/2020/10/gpt3-travel-agent"
    ).data
    assert (a.id is not None)
    assert (a.name == name_a)
    assert (a.corpusId == corpus.id)

    resp = File.list(client, corpusId=corpus.id).data
    assert (len(resp.files) == 2)


def test_corpus_upsert():
    client = _steamship()
    name = _random_name()
    corpus1 = Corpus.create(client, name=name).data
    assert (corpus1.id is not None)

    #This finds the same corpus as corpus1
    corpus2 = Corpus.create(client, name=name)
    assert (corpus2.data is not None)
    assert (corpus2.error is None)
    #Commenting out below, which deletes corpus1, which results in the corpus3 test failing
    #corpus2.data.delete()

    corpus2 = Corpus.create(client, name=name, handle=_random_name()).data
    assert (corpus1.id != corpus2.id)
    assert (corpus2.id is not None)
    corpus2.delete()

    assert (corpus1.handle is not None)
    assert (len(corpus1.handle) > 0)
    corpus3 = Corpus.create(client, handle=corpus1.handle, upsert=True)
    assert (corpus3.error is None)
    assert (corpus3.data.id is not None)
    assert (corpus1.id == corpus3.data.id)
    corpus1.delete()


def test_corpus_delete():
    client = _steamship()
    name = _random_name()
    corpus1 = Corpus.create(client, name=name).data
    resp = corpus1.delete()
    assert (resp.error is None)
    assert (resp.data is not None)
    assert (resp.data.id == corpus1.id)

    name_a = "{}.mkd".format(_random_name())
    a = File.create(
        client,
        name=name_a,
        content="A",
        corpusId=corpus1.id,
        mimeType=MimeTypes.MKD
    )
    assert (a.data is None)
    assert (a.error is not None)


def test_corpus_delete_cascade():
    client = _steamship()
    name = _random_name()

    corpus1 = Corpus.create(client=client, name=name).data

    name_a = "{}.mkd".format(_random_name())
    a = File.create(
        client=client,
        corpusId=corpus1.id,
        name=name_a,
        content="A",
        mimeType=MimeTypes.MKD
    ).data

    res = a.query()
    corpus1.delete()

    # Now verify the file isn't there!
    aa = File.get(client, id=a.id)
    assert (aa.data is None)
    assert (aa.error is not None)

# TODO: Add tests w/ operations in different spaces



import dataclasses
import io
import logging
from typing import Dict, Type

from steamship.app.app import App
from steamship.app.request import Request
from steamship.app.response import Response, Error, Http
from steamship.client.client import Steamship
from steamship.base import SteamshipError

def create_handler(App: Type[App]):
    """Wrapper function for an Steamship app within an AWS Lambda function.
    """

    def handler(event: Dict, context: Dict = None) -> Dict:
        # Create a new Steamship client
        client = Steamship(
            configDict=event.get("clientConfig", None)
        )

        app = None

        try:
            request = Request.from_dict(event)
        except Exception as ex:
            logging.exception("Unable to parse inbound request")
            response = Error(
                message="There was an exception thrown handling this request.",
                error=ex
            )

        try:
            app = App(client=client, config=request.invocation.config)
        except Exception as ex:
            logging.exception("Unable to initialize app.")
            response = Error(
                message="There was an exception thrown handling this request.",
                error=ex
            )


        try:
            if app is not None:
                response = app(request)
        except Exception as ex:
            logging.exception("Unable to run app method.")
            response = Error(
                message="There was an exception thrown handling this request.",
                error=ex
            )

        lambda_response: Response = None

        if type(response) == Response:
            lambda_response = response
        elif type(response) == SteamshipError:
            lambda_response = Response(
                error=response,
                http=Http(status=500)
            )
        elif type(response) == io.BytesIO:
            lambda_response = Response(bytes=response)
        elif type(response) == dict:
            lambda_response = Response(json=response)
        elif type(response) == str:
            lambda_response = Response(string=response)
        elif type(response) in [float, int, bool]:
            lambda_response = Response(json=response)
        elif dataclasses.is_dataclass(response):
            lambda_response = Response(json=response)
        else:
            lambda_response = Response(
                error=SteamshipError(message="Handler provided unknown response type."),
                http=Http(status=500)
            )
        if lambda_response is None:
            lambda_response = Response(
                error=SteamshipError(message="Handler provided no response."),
                http=Http(status=500)
            )
        return dataclasses.asdict(lambda_response)

    return handler



import json
from dataclasses import dataclass
from typing import List, Any

from steamship.base import Client, Request, Response


@dataclass
class Tag:
    client: Client = None
    id: str = None
    fileId: str = None
    blockId: str = None
    kind: str = None  # E.g. ner
    name: str = None  # E.g. person
    value: str = None  # JSON Metadata
    startIdx: int = None  # w/r/t block.text. None means 0 if blockId is not None
    endIdx: int = None  # w/r/t block.text. None means -1 if blockId is not None

    @dataclass
    class CreateRequest(Request):
        id: str = None
        fileId: str = None
        blockId: str = None
        kind: str = None
        name: str = None
        startIdx: int = None
        endIdx: int = None
        value: Any = None
        upsert: bool = None

        @staticmethod
        def from_dict(d: any, client: Client = None) -> "Tag.CreateRequest":
            return Tag.CreateRequest(
                id=d.get('id', None),
                fileId=d.get('fileId', None),
                blockId=d.get('blockId', None),
                kind=d.get('kind', None),
                name=d.get('name', None),
                startIdx=d.get('startIdx', None),
                endIdx=d.get('endIdx', None),
                value=d.get('value', None),
                upsert=d.get('upsert', None),
            )

    @dataclass
    class DeleteRequest(Request):
        id: str = None
        fileId: str = None
        blockId: str = None

    @dataclass
    class ListRequest(Request):
        fileId: str = None
        blockId: str = None

    @dataclass
    class ListResponse(Request):
        tags: List["Tag"] = None

        @staticmethod
        def from_dict(d: any, client: Client = None) -> "Tag.ListResponse":
            if d is None:
                return None
            return Tag.ListResponse(
                tags=[Tag.from_dict(x, client=client) for x in (d.get("tags", []) or [])]
            )

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "Tag":
        return Tag(
            client=client,
            id=d.get('id', None),
            fileId=d.get('fileId', None),
            blockId=d.get('blockId', None),
            kind=d.get('kind', None),
            name=d.get('name', None),
            startIdx=d.get('startIdx', None),
            endIdx=d.get('endIdx', None),
            value=d.get('value', None)
        )

    def to_dict(self) -> dict:
        return dict(
            id=self.id,
            fileId=self.fileId,
            blockId=self.blockId,
            kind=self.kind,
            name=self.name,
            startIdx=self.startIdx,
            endIdx=self.endIdx,
            value=self.value
        )

    @staticmethod
    def create(
            client: Client,
            fileId: str = None,
            blockId: str = None,
            kind: str = None,
            name: str = None,
            startIdx: int = None,
            endIdx: int = None,
            value: Any = None,
            upsert: bool = None,
            spaceId: str = None,
            spaceHandle: str = None
    ) -> Response["Tag"]:
        if isinstance(value, dict) or isinstance(value, list):
            value = json.dumps(value)

        req = Tag.CreateRequest(
            fileId=fileId,
            blockId=blockId,
            kind=kind,
            name=name,
            startIdx=startIdx,
            endIdx=endIdx,
            value=value,
            upsert=upsert
        )
        return client.post(
            'tag/create',
            req,
            expect=Tag,
            spaceId=spaceId,
            spaceHandle=spaceHandle
        )

    @staticmethod
    def listPublic(
            client: Client,
            fileId: str = None,
            blockId: str = None,
            spaceId: str = None,
            spaceHandle: str = None
    ) -> Response["Tag.ListResponse"]:
        return client.post(
            'tag/list',
            Tag.ListRequest(
                fileId=fileId,
                blockId=blockId
            ),
            expect=Tag.ListResponse,
            spaceId=spaceId,
            spaceHandle=spaceHandle
        )

    def delete(self) -> Response["Tag"]:
        return self.client.post(
            'tag/delete',
            Tag.DeleteRequest(id=self.id, fileId=self.fileId, blockId=self.blockId),
            expect=Tag,
        )



from dataclasses import dataclass
from typing import List, Dict
from steamship.base import Client


@dataclass
class EmbedRequest:
    docs: List[str]
    pluginInstance: str
    metadata: Dict = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "EmbedRequest":
        return EmbedRequest(
            docs=d.get('docs', None),
            pluginInstance=d.get('pluginInstance', None),
            metadata=d.get('metadata', {})
        )


import json
from dataclasses import dataclass
from typing import List, Dict, Union, TypeVar, Generic

from steamship.base import Client, Request, Response, metadata_to_str
from steamship.data.search import Hit


@dataclass
class EmbedAndSearchRequest(Request):
    query: str
    docs: List[str]
    pluginInstance: str
    k: int = 1


#TODO: These types are not generics like the Swift QueryResult/QueryResults.
@dataclass
class QueryResult():
    value: Hit
    score: float
    index: int
    id: str

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "QueryResult":
        value = Hit.from_dict(d.get("value", {}))
        return QueryResult(
            value = value,
            score = d.get('score'),
            index = d.get('index'),
            id = d.get('id')
        )

@dataclass
class QueryResults(Request):
    items: List[QueryResult] = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "QueryResults":
        items = [QueryResult.from_dict(h) for h in (d.get("items", []) or [])]
        return QueryResults(
            items=items
        )





@dataclass
class EmbeddedItem:
    id: str = None
    indexId: str = None
    fileId: str = None
    blockId: str = None
    tagId: str = None
    value: str = None
    externalId: str = None
    externalType: str = None
    metadata: any = None
    embedding: List[float] = None

    def clone_for_insert(self) -> "EmbeddedItem":
        """Produces a clone with a string representation of the metadata"""
        ret = EmbeddedItem(
            id=self.id,
            indexId=self.indexId,
            fileId=self.fileId,
            blockId=self.blockId,
            tagId=self.tagId,
            value=self.value,
            externalId=self.externalId,
            externalType=self.externalType,
            metadata=self.metadata,
            embedding=self.embedding
        )
        if isinstance(ret.metadata, dict) or isinstance(ret.metadata, list):
            ret.metadata = json.dumps(ret.metadata)
        return ret

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "EmbeddedItem":
        return EmbeddedItem(
            id=d.get('id', None),
            indexId=d.get('indexId', None),
            fileId=d.get('fileId', None),
            blockId=d.get('blockId', None),
            tagId=d.get('tagId', None),
            value=d.get('value', None),
            externalId=d.get('externalId', None),
            externalType=d.get('externalType', None),
            metadata=d.get('metadata', None),
            embedding=d.get('embedding', None),
        )

@dataclass
class IndexCreateRequest(Request):
    handle: str = None
    name: str = None
    pluginInstance: str = None
    upsert: bool = True
    externalId: str = None
    externalType: str = None
    metadata: any = None



@dataclass
class IndexInsertRequest(Request):
    indexId: str
    items: List[EmbeddedItem] = None
    value: str = None
    fileId: str = None
    blockType: str = None
    externalId: str = None
    externalType: str = None
    metadata: any = None
    reindex: bool = True


@dataclass
class IndexItemId:
    indexId: str = None
    id: str = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "IndexItemId":
        return IndexItemId(
            indexId=d.get('indexId', None),
            id=d.get('id', None)
        )


@dataclass
class IndexInsertResponse:
    itemIds: List[IndexItemId] = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "IndexInsertResponse":
        return IndexInsertResponse(
            itemIds=[IndexItemId.from_dict(x) for x in (d.get('itemIds', []) or [])]
        )


@dataclass
class IndexEmbedRequest(Request):
    id: str


@dataclass
class IndexEmbedResponse:
    id: str

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "IndexEmbedResponse":
        return IndexEmbedResponse(
            id=d.get('id', None)
        )


@dataclass
class IndexSearchRequest(Request):
    id: str
    query: str = None
    queries: List[str] = None
    k: int = 1
    includeMetadata: bool = False


@dataclass
class IndexSnapshotRequest(Request):
    indexId: str
    # This variable is intended only to support
    # unit testing.
    windowSize: int = None


@dataclass
class IndexSnapshotResponse:
    id: str
    snapshotId: str

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "IndexSnapshotResponse":
        return IndexSnapshotResponse(
            id=d.get('id', None),
            snapshotId=d.get('snapshotId', None)
        )


@dataclass
class ListSnapshotsRequest(Request):
    id: str = None


@dataclass
class ListSnapshotsResponse:
    snapshots: List[IndexSnapshotResponse]

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "IndexSnapshotResponse":
        return IndexSnapshotResponse(
            snapshots=[IndexSnapshotResponse.from_dict(dd) for dd in (d.get('snapshots', []) or [])]
        )


@dataclass
class ListItemsRequest(Request):
    id: str = None
    fileId: str = None
    blockId: str = None
    spanId: str = None


@dataclass
class ListItemsResponse:
    items: List[EmbeddedItem]

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "ListItemsResponse":
        return ListItemsResponse(
            items=[EmbeddedItem.from_dict(dd) for dd in (d.get('items', []) or [])]
        )


@dataclass
class DeleteSnapshotsRequest(Request):
    snapshotId: str = None


@dataclass
class DeleteSnapshotsResponse(Request):
    snapshotId: str = None

@dataclass
class DeleteEmbeddingIndexRequest(Request):
    id: str

@dataclass
class EmbeddingIndex:
    """A persistent, read-optimized index over embeddings.
    """

    client: Client = None
    id: str = None
    handle: str = None
    name: str = None
    plugin: str = None
    externalId: str = None
    externalType: str = None
    metadata: str = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "EmbeddingIndex":
        if 'embeddingIndex' in d:
            d = d['embeddingIndex']
        elif 'index' in d:
            d = d['index']
        return EmbeddingIndex(
            client=client,
            id=d.get('id', None),
            handle=d.get('handle', None),
            name=d.get('name', None),
            plugin=d.get('plugin', None),
            externalId=d.get('externalId', None),
            externalType=d.get('externalType', None),
            metadata=d.get('metadata', None)
        )

    def insert_file(
            self,
            fileId: str,
            blockType: str = None,
            externalId: str = None,
            externalType: str = None,
            metadata: Union[int, float, bool, str, List, Dict] = None,
            reindex: bool = True,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None
    ) -> Response[IndexInsertResponse]:
        if isinstance(metadata, dict) or isinstance(metadata, list):
            metadata = json.dumps(metadata)

        req = IndexInsertRequest(
            indexId=self.id,
            fileId=fileId,
            blockType=blockType,
            externalId=externalId,
            externalType=externalType,
            metadata=metadata,
            reindex=reindex,
        )
        return self.client.post(
            'embedding-index/item/create',
            req,
            expect=IndexInsertResponse,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def insert_many(
            self,
            items: List[Union[EmbeddedItem, str]],
            reindex: bool = True,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None
    ) -> Response[IndexInsertResponse]:
        newItems = []
        for item in items:
            if type(item) == str:
                newItems.append(EmbeddedItem(value=item))
            else:
                newItems.append(item)

        req = IndexInsertRequest(
            indexId=self.id,
            value=None,
            items=[item.clone_for_insert() for item in newItems],
            reindex=reindex,
        )
        return self.client.post(
            'embedding-index/item/create',
            req,
            expect=IndexInsertResponse,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def insert(
            self,
            value: str,
            externalId: str = None,
            externalType: str = None,
            metadata: Union[int, float, bool, str, List, Dict] = None,
            reindex: bool = True,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None
    ) -> Response[IndexInsertResponse]:

        req = IndexInsertRequest(
            indexId=self.id,
            value=value,
            items=None,
            externalId=externalId,
            externalType=externalType,
            metadata=metadata_to_str(metadata),
            reindex=reindex,
        )
        return self.client.post(
            'embedding-index/item/create',
            req,
            expect=IndexInsertResponse,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def embed(
            self,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None) -> Response[IndexEmbedResponse]:
        req = IndexEmbedRequest(
            self.id
        )
        return self.client.post(
            'embedding-index/embed',
            req,
            expect=IndexEmbedResponse,
            asynchronous=True,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def create_snapshot(
            self,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None) -> Response[IndexSnapshotResponse]:
        req = IndexSnapshotRequest(
            indexId=self.id
        )
        return self.client.post(
            'embedding-index/snapshot/create',
            req,
            expect=IndexSnapshotResponse,
            asynchronous=True,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def list_snapshots(
            self,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None) -> Response[ListSnapshotsResponse]:
        req = ListSnapshotsRequest(
            id=self.id
        )
        return self.client.post(
            'embedding-index/snapshot/list',
            req,
            expect=ListSnapshotsResponse,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def list_items(
            self,
            fileId: str = None,
            blockId: str = None,
            spanId: str = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None) -> Response[ListItemsResponse]:
        req = ListItemsRequest(
            id=self.id,
            fileId=fileId,
            blockId=blockId,
            spanId=spanId
        )
        return self.client.post(
            'embedding-index/item/list',
            req,
            expect=ListItemsResponse,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def delete_snapshot(
            self,
            snapshot_id: str,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None) -> Response[DeleteSnapshotsResponse]:
        req = DeleteSnapshotsRequest(
            snapshotId=snapshot_id
        )
        return self.client.post(
            'embedding-index/snapshot/delete',
            req,
            expect=DeleteSnapshotsResponse,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def delete(
            self,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None) -> "Response[EmbeddingIndex]":
        return self.client.post(
            'embedding-index/delete',
            DeleteEmbeddingIndexRequest(id=self.id),
            expect=EmbeddingIndex,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def search(
            self,
            query: Union[str, List[str]],
            k: int = 1,
            includeMetadata: bool = False,
            pd=False,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None
    ) -> Response[QueryResults]:
        if type(query) == list:
            req = IndexSearchRequest(
                self.id,
                query=None,
                queries=query,
                k=k,
                includeMetadata=includeMetadata,
            )
        else:
            req = IndexSearchRequest(
                self.id,
                query=query,
                queries=None,
                k=k,
                includeMetadata=includeMetadata,
            )
        ret = self.client.post(
            'embedding-index/search',
            req,
            expect=QueryResults,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

        if pd is False:
            return ret

        import pandas as pd  # type: ignore
        return pd.DataFrame([(hit.score, hit.value) for hit in ret.data.hits], columns=['Score', 'Value'])

    @staticmethod
    def create(
            client: Client,
            handle: str = None,
            name: str = None,
            pluginInstance: str = None,
            upsert: bool = True,
            externalId: str = None,
            externalType: str = None,
            metadata: any = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None
    ) -> "Response[EmbeddingIndex]":
        req = IndexCreateRequest(
            handle=handle,
            name=name,
            pluginInstance=pluginInstance,
            upsert=upsert,
            externalId=externalId,
            externalType=externalType,
            metadata=metadata,
        )
        return client.post(
            'embedding-index/create',
            req,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space,
            expect=EmbeddingIndex
        )



from steamship.app import get, App, Response, post, create_handler


class HelloWorld(App):
    @post('greet')
    def greet(self, name: str = "Person") -> Response:
        greeting = self.config['greeting']
        return Response(string='{}, {}'.format(greeting, name))


handler = create_handler(HelloWorld)



from dataclasses import dataclass
from typing import Dict

from steamship.base import Client, Request
from steamship.base.response import Response


class PluginInstance:
    pass


@dataclass
class CreatePluginInstanceRequest(Request):
    id: str = None
    pluginId: str = None
    pluginHandle: str = None
    pluginVersionId: str = None
    pluginVersionHandle: str = None
    name: str = None
    handle: str = None
    upsert: bool = None
    config: Dict[str, any] = None


@dataclass
class DeletePluginInstanceRequest(Request):
    id: str


@dataclass
class PluginInstance:
    client: Client = None
    id: str = None
    name: str = None
    handle: str = None
    pluginId: str = None
    pluginVersionId: str = None
    userId: str = None
    config: Dict[str, any] = None
    spaceId: str = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "PluginInstance":
        if 'pluginInstance' in d:
            d = d['pluginInstance']

        return PluginInstance(
            client=client,
            id=d.get('id', None),
            name=d.get('name', None),
            handle=d.get('handle', None),
            pluginId=d.get('pluginId', None),
            pluginVersionId=d.get('pluginVersionId', None),
            config=d.get('config', None),
            userId=d.get('userId', None)
        )

    @staticmethod
    def create(
            client: Client,
            pluginId: str = None,
            pluginHandle: str = None,
            pluginVersionId: str = None,
            pluginVersionHandle: str = None,
            name: str = None,
            handle: str = None,
            upsert: bool = None,
            config: Dict[str, any] = None
    ) -> Response[PluginInstance]:
        req = CreatePluginInstanceRequest(
            name=name,
            handle=handle,
            pluginId=pluginId,
            pluginHandle=pluginHandle,
            pluginVersionId=pluginVersionId,
            pluginVersionHandle=pluginVersionHandle,
            upsert=upsert,
            config=config
        )

        return client.post(
            'plugin/instance/create',
            payload=req,
            expect=PluginInstance
        )

    def delete(self) -> PluginInstance:
        req = DeletePluginInstanceRequest(
            id=self.id
        )
        return self.client.post(
            'plugin/instance/delete',
            payload=req,
            expect=PluginInstance
        )


@dataclass
class ListPrivatePluginInstancesRequest(Request):
    pass



class DocTag:
    """This class provides a set of Markdown-style constants to use with Tag(kind="doc")
    """
    doc = "doc"
    page = "page"  # E.g. in a PDF
    region = "region"  # E.g., abstract catchall region in a document
    header = "header"
    h1 = "h1"
    h2 = "h2"
    h3 = "h3"
    h4 = "h4"
    h5 = "h5"
    line = "line"
    title = "title"
    subtitle = "subtitle"
    footer = "footer"
    paragraph = "paragraph"
    list = "list"
    blockquote = "blockquote"
    blockcode = "blockcode"
    unk = "unk"
    sentence = "sentence"
    token = "token"
    span = "span"



from steamship import File, Corpus, Plugin, PluginVersion, PluginInstance

from ..client.helpers import deploy_plugin, _steamship, _random_corpus
from ..demo_apps.plugin_file_importer import TEST_DOC

__copyright__ = "Steamship"
__license__ = "MIT"


def test_e2e_corpus_importer():
    client = _steamship()
    testFileImporterInstance = PluginInstance.create(client, pluginHandle='test-fileImporter-valueOrData', upsert=True).data
    with deploy_plugin("plugin_corpus_importer.py", "corpusImporter") as (plugin, version, instance):
        #with PluginVersion.(client, "corpusImporter", "do_import", app, instance) as plugin:
        with _random_corpus(client) as corpus:
            resp = corpus.doImport(
                pluginInstance=instance.handle,
                fileImporterPluginInstance=testFileImporterInstance.handle,
                value="dummy-value"
            )
            resp.wait()

            # We should now have two files!
            files = File.list(client, corpusId=corpus.id).data
            assert (files.files is not None)
            assert (len(files.files) == 2)

            for file in files.files:
                data = file.raw().data
                assert (data.decode('utf-8') == TEST_DOC)
                file.delete()



import json
from dataclasses import dataclass
from typing import Any, List

from steamship.base import Client, Request, Response, str_to_metadata
from steamship.base.request import IdentifierRequest


@dataclass
class Corpus:
    """A corpus of files.
    """
    client: Client
    id: str = None
    name: str = None
    handle: str = None
    description: str = None
    externalId: str = None
    externalType: str = None
    metadata: Any = None

    @dataclass
    class CreateRequest(Request):
        corpusId: str = None
        name: str = None
        handle: str = None
        description: str = None
        externalId: str = None
        externalType: str = None
        isPublic: bool = None
        metadata: str = None
        upsert: bool = None

    @dataclass
    class DeleteRequest(Request):
        corpusId: str

    @dataclass
    class ListRequest(Request):
        pass

    @dataclass
    class ListResponse(Request):
        corpora: List["Corpus"]

        @staticmethod
        def from_dict(d: any, client: Client = None) -> "Corpus.ListResponse":
            return Corpus.ListResponse(
                plugins=[Corpus.from_dict(x) for x in (d.get("corpus", []) or [])]
            )

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "Corpus":
        if 'corpus' in d:
            d = d['corpus']

        return Corpus(
            client=client,
            id=d.get('id', None),
            name=d.get('name', None),
            handle=d.get('handle', None),
            description=d.get('description', None),
            externalId=d.get('externalId', None),
            externalType=d.get('externalType', None),
            metadata=str_to_metadata(d.get("metadata", None)),
        )

    @staticmethod
    def default(
            client: Client,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None
    ) -> "Response[Corpus]":
        req = IdentifierRequest()
        return client.post(
            'corpus/get',
            req,
            expect=Corpus,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    @staticmethod
    def create(
            client: Client,
            name: str = None,
            handle: str = None,
            description: str = None,
            externalId: str = None,
            externalType: str = None,
            metadata: any = None,
            isPublic: bool = False,
            upsert: bool = True,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None
    ) -> "Response[Corpus]":
        if isinstance(metadata, dict) or isinstance(metadata, list):
            metadata = json.dumps(metadata)

        req = Corpus.CreateRequest(
            name=name,
            handle=handle,
            description=description,
            isPublic=isPublic,
            upsert=upsert,
            externalId=externalId,
            externalType=externalType,
            metadata=metadata
        )
        return client.post(
            'corpus/create',
            req,
            expect=Corpus,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def delete(
            self,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None) -> Response["Corpus"]:
        return self.client.post(
            'corpus/delete',
            IdentifierRequest(id=self.id),
            expect=Corpus,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )



from steamship import MimeTypes
from steamship.data.tags import Tag

from tests.client.helpers import _random_name, _steamship

__copyright__ = "Steamship"
__license__ = "MIT"


def test_file_tag():
    steamship = _steamship()
    name_a = "{}.mkd".format(_random_name())
    a = steamship.upload(
        name=name_a,
        content="A",
        mimeType=MimeTypes.MKD
    ).data
    assert (a.id is not None)
    assert (a.name == name_a)
    assert (a.mimeType == MimeTypes.MKD)

    t1 = Tag.create(steamship, fileId=a.id, name="test1").data
    t2 = Tag.create(steamship, fileId=a.id, name="test2").data

    tags = Tag.listPublic(steamship, fileId=a.id)
    assert (tags.data is not None)
    assert (tags.data.tags is not None)
    assert (len(tags.data.tags) == 2)

    must = ['test1', 'test2']
    for tag in tags.data.tags:
        assert (tag.name in must)
        must.remove(tag.name)
    assert (len(must) == 0)

    for tag in tags.data.tags:
        if tag.name == 'test1':
            tag.delete()

    tags = Tag.listPublic(steamship, fileId=a.id)
    assert (tags.data is not None)
    assert (tags.data.tags is not None)
    assert (len(tags.data.tags) == 1)

    must = ['test2']
    for tag in tags.data.tags:
        assert (tag.name in must)
        must.remove(tag.name)
    assert (len(must) == 0)

    tags.data.tags[0].delete()

    tags = Tag.listPublic(steamship, fileId=a.id)
    assert (tags.data is not None)
    assert (tags.data.tags is not None)
    assert (len(tags.data.tags) == 0)

    a.delete()



from steamship.app import get, App, Response, post, create_handler


class HelloWorld(App):
    @post('greet')
    def greet(self, name: str = "Person") -> Response:
        return Response(string='Hello, {}'.format(name))

    @get('space')
    def space(self) -> Response:
        return Response(string=self.client.config.spaceId)


handler = create_handler(HelloWorld)



import requests

from steamship import AppInstance
from steamship.data.user import User
from tests.client.helpers import _steamship, deploy_app, shouldUseSubdomain

__copyright__ = "Steamship"
__license__ = "MIT"


def test_configurable_instance_invoke():

    greeting1 = "Hola"
    configTemplate = { "greeting" : {"type": "string"}}
    instanceConfig = { "greeting" : greeting1}

    with deploy_app("configurable_hello_world.py", versionConfigTemplate=configTemplate, instanceConfig=instanceConfig) as (app, version, instance):
        # Now let's invoke it!
        # Note: we're invoking the data at configurable_hello_world.py in the tests/demo_apps folder
        client = _steamship()
        res = instance.post('greet').data
        assert (res == "{}, Person".format(greeting1))


        greeting2 = "Hallo"
        instanceConfig2 = { "greeting" : greeting2}
        instance2 = AppInstance.create(
            client,
            appId=app.id,
            appVersionId=version.id,
            config=instanceConfig2
        )
        instance2.wait()
        assert (instance2.error is None)
        assert (instance2.data is not None)
        instance2 = instance2.data

        res2 = instance2.post('greet').data
        assert (res2 == "{}, Person".format(greeting2))




from steamship import App, AppVersion

from tests.client.helpers import _random_name, _steamship, create_app_zip

__copyright__ = "Steamship"
__license__ = "MIT"


def test_version_create():
    client = _steamship()
    name = _random_name()

    app = App.create(client, name=name).data
    zip_bytes = create_app_zip('demo_app.py')

    version = AppVersion.create(
        client,
        appId=app.id,
        filebytes=zip_bytes
    )

    version.wait()

    res = version.data.delete()
    assert (res.error is None)

    res = app.delete()
    assert (res.error is None)



# Plugin
#
# This file contains the abstractions for managing Steamship plugins.
# To see how to implement a Steamship Plugin, see service.py in the same folder.
#
#

import json
from dataclasses import dataclass
from typing import List, Dict, Union

from steamship.base.client import Client
from steamship.base.request import Request
from steamship.base.response import Response


class Plugin:
    pass


@dataclass
class CreatePluginRequest(Request):
    isTrainable: bool
    id: str = None
    name: str = None
    type: str = None
    transport: str = None
    isPublic: bool = None
    isTrainable: bool = None
    handle: str = None
    description: str = None
    dimensionality: int = None
    limitAmount: int = None
    limitUnit: str = None
    apiKey: str = None
    metadata: str = None
    upsert: bool = None


@dataclass
class DeletePluginRequest(Request):
    id: str


@dataclass
class ListPublicPluginsRequest(Request):
    type: str


@dataclass
class ListPrivatePluginsRequest(Request):
    type: str


@dataclass
class ListPluginsResponse(Request):
    plugins: List[Plugin]

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "ListPluginsResponse":
        return ListPluginsResponse(
            plugins=[Plugin.from_dict(x, client=client) for x in (d.get("plugins", []) or [])]
        )


@dataclass
class GetPluginRequest(Request):
    type: str = None
    id: str = None
    handle: str = None


class PluginType:
    parser = "parser"
    classifier = "classifier"
    tagger = "tagger"


class PluginAdapterType:
    steamshipDocker = "steamshipDocker"
    steamshipSagemaker = "steamshipSagemaker"
    huggingface = "huggingface"
    openai = "openai"


class PluginTargetType:
    file = "file"
    corpus = "corpus"
    space = "space"


class LimitUnit:
    words = "words"
    characters = "characters"
    bytes = "bytes"


@dataclass
class Plugin:
    client: Client = None
    id: str = None
    name: str = None
    type: str = None
    transport: str = None
    isPublic: bool = None
    isTrainable: bool = None
    handle: str = None
    description: str = None
    dimensionality: int = None
    limitAmount: int = None
    limitUnit: str = None
    apiKey: str = None
    metadata: str = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "Plugin":
        if 'plugin' in d:
            d = d['plugin']

        return Plugin(
            client=client,
            id=d.get('id', None),
            name=d.get('name', None),
            type=d.get('type', None),
            transport=d.get('transport', None),
            isPublic=d.get('isPublic', None),
            isTrainable=d.get('isTrainable', None),
            handle=d.get('handle', None),
            description=d.get('description', None),
            dimensionality=d.get('dimensionality', None),
            limitAmount=d.get('limitAmount', None),
            limitUnit=d.get('limitUnit', None),
            apiKey=d.get('apiKey', None),
            metadata=d.get('metadata', None)
        )

    @staticmethod
    def create(
            client: Client,
            isTrainable: bool,
            name: str,
            description: str,
            type: str,
            transport: str,
            isPublic: bool,
            isTrainable: bool = False,
            handle: str = None,
            dimensionality: int = None,
            limitAmount: int = None,
            limitUnit: str = None,
            apiKey: str = None,
            metadata: Union[str, Dict, List] = None,
            upsert: bool = None,
            spaceId: str = None,
            spaceHandle: str = None
    ) -> Response[Plugin]:
        if isinstance(metadata, dict) or isinstance(metadata, list):
            metadata = json.dumps(metadata)

        req = CreatePluginRequest(
            isTrainable=isTrainable,
            name=name,
            type=type,
            transport=transport,
            isPublic=isPublic,
            isTrainable=isTrainable,
            handle=handle,
            description=description,
            dimensionality=dimensionality,
            limitAmount=limitAmount,
            limitUnit=limitUnit,
            apiKey=apiKey,
            metadata=metadata,
            upsert=upsert
        )
        return client.post(
            'plugin/create',
            req,
            expect=Plugin,
            spaceId=spaceId,
            spaceHandle=spaceHandle
        )

    @staticmethod
    def listPublic(
            client: Client,
            type: str = None,
            spaceId: str = None,
            spaceHandle: str = None
    ) -> Response[ListPluginsResponse]:
        return client.post(
            'plugin/public',
            ListPublicPluginsRequest(type=type),
            expect=ListPluginsResponse,
            spaceId=spaceId,
            spaceHandle=spaceHandle
        )

    @staticmethod
    def listPrivate(
            client: Client,
            type=None,
            spaceId: str = None,
            spaceHandle: str = None
    ) -> Response[ListPluginsResponse]:
        return client.post(
            'plugin/private',
            ListPrivatePluginsRequest(type=type),
            expect=ListPluginsResponse,
            spaceId=spaceId,
            spaceHandle=spaceHandle
        )

    @staticmethod
    def get(client: Client, handle: str):
        return client.post(
            'plugin/get',
            GetPluginRequest(handle=handle),
            expect=Plugin
        )

    @staticmethod
    def getPublic(client: Client, handle: str):
        publicPlugins = Plugin.listPublic(client=client).data.plugins
        for plugin in publicPlugins:
            if plugin.handle == handle:
                return plugin
        return None

    def delete(self) -> Response[Plugin]:
        return self.client.post(
            'plugin/delete',
            DeletePluginRequest(id=self.id),
            expect=Plugin,
        )



from steamship.plugin.inputs.raw_data_plugin_input import RawDataPluginInput
from steamship.plugin.service import PluginRequest

from ..demo_apps.plugin_converter import TestConverterPlugin

__copyright__ = "Steamship"
__license__ = "MIT"


def test_resp():
    converter = TestConverterPlugin()
    request = PluginRequest(data=RawDataPluginInput(data="Hi there"))
    response = converter.run(request).data
    assert (response.file is not None)
    assert (len(response.file.blocks) == 4)

    # Now test as an HTTP Call



import json
from dataclasses import dataclass, asdict
from typing import List

from steamship import File
from steamship.base import Client


@dataclass
class TagRequest:
    type: str = None
    id: str = None
    name: str = None
    handle: str = None
    pluginInstance: str = None
    file: File.CreateRequest = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "TagRequest":

        return TagRequest(
            type = d.get('type', None),
            id = d.get('id', None),
            name = d.get('name', None),
            handle = d.get('handle', None),
            pluginInstance= d.get('pluginInstance', None),
            file= File.CreateRequest.from_dict(d.get('file', {}))
        )

    def to_dict(self) -> dict:
        return asdict(self)

@dataclass
class TagResponse:
    file: File = None

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "TagResponse":
        return TagResponse(
            file= File.from_dict(d.get('file', {}))
        )

    def to_dict(self) -> dict:
        return asdict(self)



from steamship.data.file import File
from steamship.plugin.service import PluginRequest, PluginResponse

__copyright__ = "Steamship"
__license__ = "MIT"

from ..demo_apps.plugin_file_importer import TestFileImporterPlugin, TEST_DOC

TEST_REQ = File.CreateRequest(
    value="Hi there."
)
TEST_PLUGIN_REQ = PluginRequest(data=TEST_REQ)
TEST_REQ_DICT = TEST_PLUGIN_REQ.to_dict()


def _test_resp(res):
    assert (type(res) == PluginResponse)
    assert (type(res.data) == File.CreateResponse)
    assert (res.data.data == TEST_DOC)


def test_importer():
    importer = TestFileImporterPlugin()
    res = importer.run(TEST_PLUGIN_REQ)
    _test_resp(res)



from typing import Dict

from steamship import File
from steamship.base import Client

from dataclasses import dataclass

@dataclass
class BlockAndTagPluginInput:
    file: File = None

    @staticmethod
    def from_dict(d: any = None, client: Client = None) -> "BlockAndTagPluginInput":
        if d is None:
            return None

        return BlockAndTagPluginInput(
            file=File.from_dict(d.get('file', None), client=client)
        )

    def to_dict(self) -> Dict:
        if self.file is None:
            return dict()
        return dict(file=self.file.to_dict())




from steamship import MimeTypes, DocTag, PluginInstance, Block
from steamship.base import Client
from steamship.base.response import TaskStatus

from tests.client.helpers import _random_name, _steamship

__copyright__ = "Steamship"
__license__ = "MIT"


# TODO: It should fail if the docs field is empty.
# TODO: It should fail if the file hasn't been converted.

def count_blocks_with_tag(blocks: [Block], tag_kind: str, tag_name: str):
    c = 0
    for block in blocks:
        if any([tag.kind == tag_kind and tag.name == tag_name for tag in block.tags]):
            c += 1
    return c

def count_tags(blocks: [Block], tag_kind: str, tag_name: str):
    c = 0
    for block in blocks:
        tag_matches = [1 if tag.kind == tag_kind and tag.name == tag_name else 0 for tag in block.tags]
        c += sum(tag_matches)
    return c

def parse_file(client: Client, parserInstanceHandle: str):
    name_a = "{}.mkd".format(_random_name())
    T = "A Poem"
    P1_1 = "Roses are red."
    P1_2 = "Violets are blue."
    P2_1 = "Sugar is sweet, and I love you."

    CONTENT = "# {}\n\n{} {}\n\n{}".format(T, P1_1, P1_2, P2_1)

    a = client.upload(
        name=name_a,
        content=CONTENT,
        mimeType=MimeTypes.MKD
    ).data
    assert (a.id is not None)
    assert (a.name == name_a)
    assert (a.mimeType == MimeTypes.MKD)

    a.convert(pluginInstance="markdown-converter-default-1.0").wait()

    raw = a.raw()
    assert (raw.data.decode('utf-8') == CONTENT)


    # The following tests should be updated once the Tag query basics are merged.
    # Instead of querying and filtering, do a query with a tag filter
    q1 = a.query().data
    assert( count_blocks_with_tag(q1.blocks, DocTag.doc, DocTag.h1) == 1)
    assert (q1.blocks[0].text == T)

    # Instead of re-filtering previous result, do a new tag filter query
    assert (count_blocks_with_tag(q1.blocks, DocTag.doc, DocTag.paragraph) == 2)
    assert (q1.blocks[1].text == "{} {}".format(P1_1, P1_2))

    # The sentences aren't yet parsed out!
    # Instead of re-filtering again, do a new tag filter query
    assert (count_blocks_with_tag(q1.blocks, DocTag.doc, DocTag.sentence) == 0)

    # Now we parse
    task = a.tag(pluginInstance=parserInstanceHandle)
    assert (task.error is None)
    assert (task.task is not None)
    assert (task.task.state == TaskStatus.waiting)

    task.wait()
    assert (task.error is None)
    assert (task.task is not None)
    assert (task.task.state == TaskStatus.succeeded)

    # Now the sentences should be parsed!
    # Again, should rewrite these once tag queries are integrated
    q2 = a.query().data
    assert (count_tags(q2.blocks, DocTag.doc, DocTag.sentence) == 4)

    a.clear()

    q2 = a.query().data
    assert (len(q2.blocks) == 0)

    a.delete()


def test_parse_file():
    steamship = _steamship()
    parser = PluginInstance.create(steamship, pluginHandle='test-tagger').data
    parse_file(steamship, parser.handle)



import pytest

from steamship.data.plugin import PluginAdapterType
from steamship.data.plugin import PluginType, Plugin
from tests.client.helpers import _random_name, _steamship

__copyright__ = "Steamship"
__license__ = "MIT"


def test_plugin_create():
    steamship = _steamship()
    name = _random_name()

    my_plugins = Plugin.listPrivate(steamship).data
    orig_count = len(my_plugins.plugins)

    # Should require name
    with pytest.raises(Exception):
        index = Plugin.create(
            client=steamship,
            description="This is just for test",
            type=PluginType.embedder,
            transport=PluginAdapterType.steamshipDocker,
            isPublic=True
        )

    # Should require description
    with pytest.raises(Exception):
        index = Plugin.create(
            client=steamship,
            name="Test Plugin",
            type=PluginType.embedder,
            transport=PluginAdapterType.steamshipDocker,
            isPublic=True
        )

    # Should require plugin type
    with pytest.raises(Exception):
        index = Plugin.create(
            client=steamship,
            name="Test Plugin",
            description="This is just for test",
            transport=PluginAdapterType.steamshipDocker,
            isPublic=True
        )

    # Should require adapter type
    with pytest.raises(Exception):
        index = steamship.plugins.create(
            client=steamship,
            name="Test Plugin",
            description="This is just for test",
            type=PluginType.embedder,
            url="http://foo4",
            isPublic=True
        )

    # Should require is public
    with pytest.raises(Exception):
        index = steamship.plugins.create(
            client=steamship,
            name="Test Plugin",
            description="This is just for test",
            type=PluginType.embedder,
            transport=PluginAdapterType.steamshipDocker,
        )

    my_plugins = Plugin.listPrivate(steamship).data
    assert (len(my_plugins.plugins) == orig_count)

    plugin = Plugin.create(
        client=steamship,
        isTrainable=False,
        name=_random_name(),
        description="This is just for test",
        type=PluginType.tagger,
        transport=PluginAdapterType.steamshipDocker,
        isPublic=False
    ).data
    my_plugins = Plugin.listPrivate(steamship).data
    assert (len(my_plugins.plugins) == orig_count + 1)

    # No upsert doesn't work
    pluginX = Plugin.create(
        client=steamship,
        isTrainable=False,
        handle=plugin.handle,
        name=plugin.name,
        description="This is just for test",
        type=PluginType.tagger,
        transport=PluginAdapterType.steamshipDocker,
        isPublic=False
    )
    assert (pluginX.error is not None)
    assert (pluginX.data is None)

    # Upsert does work
    plugin2 = Plugin.create(
        client=steamship,
        isTrainable=False,
        name=plugin.name,
        description="This is just for test 2",
        type=PluginType.tagger,
        transport=PluginAdapterType.steamshipDocker,
        isPublic=False,
        upsert=True
    ).data

    assert (plugin2.id == plugin.id)

    my_plugins = Plugin.listPrivate(steamship).data
    assert (len(my_plugins.plugins) == orig_count + 1)

    assert (plugin2.id in [plugin.id for plugin in my_plugins.plugins])
    assert (plugin2.description in [plugin.description for plugin in my_plugins.plugins])

    # assert(my_plugins.plugins[0].description != plugin.description)

    plugin.delete()

    my_plugins = Plugin.listPrivate(steamship).data
    assert (len(my_plugins.plugins) == orig_count)


def test_plugin_public():
    steamship = _steamship()
    name = _random_name()

    resp = Plugin.listPublic(steamship).data
    assert (resp.plugins is not None)
    plugins = resp.plugins

    assert (len(plugins) > 0)

    # Make sure they can't be deleted.
    res = plugins[0].delete()
    assert (res.error is not None)
    assert (res.data is None)



from steamship.extension.file import File
from steamship.data.block import Block
from steamship.plugin.service import PluginRequest
from steamship.client.operations.tagger import TagRequest

from ..client.helpers import deploy_plugin, _steamship
from tests.client.operations.test_tag_file import parse_file

__copyright__ = "Steamship"
__license__ = "MIT"


def test_e2e_parser():
    client = _steamship()
    with deploy_plugin("plugin_parser.py", "tagger") as (plugin, version, instance):
        test_doc = "Hi there"
        res = client.tag(doc= test_doc, pluginInstance=instance.handle)
        res.wait()
        assert (res.error is None)
        assert (res.data is not None)
        assert (len(res.data.file.blocks) == 1)
        assert (res.data.file.blocks[0].text == test_doc)

        # Let's try it on a file. This is the same test we run on the Swift test parser.
        # Since the python test parser is implemented to behave the same, we can reuse it!
        parse_file(client, instance.handle)



from steamship import Space

from tests.client.helpers import _steamship

__copyright__ = "Steamship"
__license__ = "MIT"


def test_default_space():
    client = _steamship()
    space = Space.get(client=client).data
    assert (space is not None)
    assert (space.handle == 'default')


def test_delete_space():
    client = _steamship()
    default = Space.get(client=client).data
    space1 = Space.create(client=client, name="Test", handle="test").data
    assert (default.id is not None)
    assert (space1.id is not None)
    assert (default.id != space1.id)

    space1.delete()
    space1a = Space.get(client=client, spaceId=space1.id)
    assert (space1a.data is None)
    assert (space1a.error is not None)

    space1 = Space.create(client=client, name="Test", handle="test").data
    assert (default.id is not None)
    assert (space1.id is not None)
    assert (default.id != space1.id)

    space1a = Space.get(client=client, spaceId=space1.id)
    assert (space1a.data is not None)
    assert (space1a.error is None)

    space1.delete()
    space1a = Space.get(client=client, spaceId=space1.id)
    assert (space1a.data is None)
    assert (space1a.error is not None)


def test_get_space():
    client = _steamship()
    default = Space.get(client=client).data
    space1 = Space.create(client=client, name="Test", handle="test").data
    space1a = Space.get(client=client, spaceId=space1.id).data
    assert (space1a.id == space1.id)
    assert (space1a.id != default.id)
    assert (space1a.handle == space1.handle)


def test_create_use_delete_space():
    client = _steamship()
    default = Space.get(client=client).data
    space1 = Space.create(client=client, name="Test", handle="test").data
    space2 = Space.create(client=client, name="Test", handle="test2").data

    assert (space1 is not None)
    assert (space1.handle == 'test')

    assert (space2 is not None)
    assert (space2.handle == 'test2')

    assert (space2.id != space1.id)
    assert (space1.id != default.id)
    assert (space2.id != default.id)

    space1a = Space.get(client=client, spaceId=space1.id).data
    space1b = Space.get(client=client, spaceHandle=space1.handle).data
    space1c = Space.get(client=client, space=space1).data

    assert (space1.id == space1a.id)
    assert (space1.id == space1b.id)
    assert (space1.id == space1c.id)

    space1ad = space1a.delete()
    assert (space1ad.data is not None)
    assert (space1ad.error is None)

    # These two are the same space! You can't delete twice!
    space1bd = space1b.delete()
    assert (space1bd.data is None)
    assert (space1bd.error is not None)
    space1cd = space1c.delete()
    assert (space1cd.data is None)
    assert (space1cd.error is not None)

    space2.delete()

    space1a_deleted = Space.get(client=client, spaceId=space1.id)
    assert (space1a_deleted.data is None)
    assert (space1a_deleted.error is not None)



from abc import ABC

from steamship.base import Client
from steamship.plugin.inputs.file_import_plugin_input import FileImportPluginInput
from steamship.plugin.outputs.raw_data_plugin_output import RawDataPluginOutput
from steamship.plugin.service import PluginService, PluginRequest




from abc import ABC, abstractmethod
from dataclasses import dataclass
from typing import TypeVar, Generic, Union, Callable

from steamship.base import Client
from steamship.base.response import SteamshipError, Task

T = TypeVar('T')
U = TypeVar('U')


@dataclass
class PluginRequest(Generic[T]):
    data: T = None

    @staticmethod
    def from_dict(
            d: any,
            subclass_request_from_dict: Callable[[dict, Client], dict] = None,
            client: Client = None
    ) -> "PluginRequest(Generic[T])":
        data = None
        if "data" in d:
            if subclass_request_from_dict is not None:
                data = subclass_request_from_dict(d["data"], client=client)
            else:
                raise SteamshipError(
                    message="No `subclass_request_from_dict` provided to parse inbound dict request."
                )
        return PluginRequest(data=data)

    def to_dict(self) -> dict:
        if self.data is None:
            return dict()
        else:
            return dict(data=self.data.to_dict())


@dataclass
class PluginResponse(Generic[U]):
    error: SteamshipError = None
    task: Task[U] = None
    data: U = None

    @staticmethod
    def from_dict(
            d: any,
            client: Client = None
    ) -> "PluginResponse(Generic[U])":
        data = None
        error = None
        task = None

        if "data" in d:
            data = d["data"].to_dict()
        if "error" in d:
            error = SteamshipError.from_dict(d["error"], client=client)
        if "task" in d:
            task = Task[U].from_dict(d["task"], client=client)

        return PluginResponse(data=data, task=task, error=error)

    def to_dict(self) -> dict:
        return dict(
            data=None if self.data is None else self.data.to_dict(),
            error=None if self.error is None else self.error.to_dict(),
            task=None if self.task is None else self.task.to_dict()
        )


class PluginService(ABC, Generic[T, U]):
    @abstractmethod
    def run(self, request: PluginRequest[T]) -> Union[U, PluginResponse[U]]:
        pass

    @classmethod
    @abstractmethod
    def subclass_request_from_dict(cls, d: any, client: Client = None) -> PluginRequest[T]:
        pass

    @classmethod
    def parse_request(cls, request: Union[PluginRequest[T], dict]) -> PluginRequest[T]:
        if type(request) == dict:
            try:
                request = PluginRequest[T].from_dict(
                    request,
                    subclass_request_from_dict=cls.subclass_request_from_dict
                )
            except Exception as error:
                raise SteamshipError(
                    message="Unable to parse input request.",
                    error=error
                )
        return request

    @classmethod
    def response_to_dict(cls, response: Union[SteamshipError, PluginResponse[U], U, dict]) -> PluginResponse[U]:
        try:
            if type(response) == PluginResponse:
                return response
            elif type(response) == SteamshipError:
                return PluginResponse(error=response)
            else:
                return PluginResponse(data=response)
        except SteamshipError as remote_error:
            return PluginResponse[U](error=remote_error)
        except Exception as error:
            return PluginResponse[U](error=SteamshipError(
                message="Unhandled exception completing your request",
                error=error
            ))



import json
from typing import Union, List, Dict

Metadata = Union[int, float, bool, str, List, Dict]


def str_to_metadata(s: str) -> Metadata:
    if s is None:
        return None
    try:
        return json.loads(s)
    except:
        s


def metadata_to_str(m: Metadata) -> str:
    if m is None:
        return None
    try:
        return json.dumps(m)
    except:
        m



from abc import ABC

from steamship.base import Client
from steamship.client.operations.corpus_importer import CorpusImportRequest, CorpusImportResponse
from steamship.plugin.service import PluginService, PluginRequest

# Note!
# =====
#
# This is the PLUGIN IMPLEMENTOR's View of a Corpus Importer.
#
# If you are using the Steamship Client, you probably want steamship.client.operations.corpus_importer instead
# of this file.
#
class CorpusImporter(PluginService[CorpusImportRequest, CorpusImportResponse], ABC):
    @classmethod
    def subclass_request_from_dict(cls, d: any, client: Client = None) -> PluginRequest[CorpusImportRequest]:
        return CorpusImportRequest.from_dict(d, client=client)



from .app import App
from .app_instance import AppInstance
from .app_version import AppVersion
from .block import Block
from .block import Block
from .corpus import Corpus
from .embeddings import EmbeddingIndex
from .file import File
from .plugin import Plugin
from .plugin_instance import PluginInstance
from .plugin_version import PluginVersion
from .space import Space
from .tags import DocTag
from .tags import Tag
from .tags import TagKind
from .tags import TextTag



from steamship import PluginInstance
from steamship.data.embeddings import EmbeddedItem

from tests.client.helpers import _random_index, _random_name, _steamship

__copyright__ = "Steamship"
__license__ = "MIT"


def _list_equal(actual, expected):
    assert len(actual) == len(expected)
    assert all([a == b for a, b in zip(actual, expected)])


def test_basic_task_comment():
    steamship = _steamship()
    name = _random_name()
    embedder = PluginInstance.create(steamship, pluginHandle='test-embedder').data
    with _random_index(steamship, embedder.handle) as index:
        item1 = EmbeddedItem(
            value="Pizza",
            externalId="pizza",
            externalType="food",
            metadata=[1, 2, 3]
        )

        index.insert(item1.value, externalId=item1.externalId, externalType=item1.externalType, metadata=item1.metadata)
        task = index.embed()
        task.wait()

        res2 = index.search(item1.value, includeMetadata=True, k=1)
        res2.add_comment(externalId="Foo", externalType="Bar", metadata=[1, 2])
        # We don't return to Res2 until the end to make sure we aren't co-mingling comments!

        res = index.search(item1.value, includeMetadata=True, k=1)

        assert (res.data.items is not None)
        assert (len(res.data.items) == 1)
        assert (res.data.items[0].value.value == item1.value)
        assert (res.data.items[0].value.externalId == item1.externalId)
        assert (res.data.items[0].value.externalType == item1.externalType)
        _list_equal(res.data.items[0].value.metadata, item1.metadata)

        res.add_comment(externalId="Foo", externalType="Bar", metadata=[1, 2])

        comments = res.list_comments()
        assert (len(comments.data.comments) == 1)

        comment = comments.data.comments[0]
        assert (comment.externalId == "Foo")
        assert (comment.externalType == "Bar")
        _list_equal(comment.metadata, [1, 2])

        res.delete_comment(comment)

        comments = res.list_comments()
        assert (len(comments.data.comments) == 0)

        # Now let's add one
        res.add_comment(externalId="Foo1", externalType="Bar1", metadata=[1, 2, 3])
        res.add_comment(externalId="Foo2", externalType="Bar2", metadata=[1, 2, 3, 4])

        comments = res.list_comments()
        assert (len(comments.data.comments) == 2)

        comment = comments.data.comments[0]
        assert (comment.externalId == "Foo1")
        assert (comment.externalType == "Bar1")
        _list_equal(comment.metadata, [1, 2, 3])

        comment = comments.data.comments[1]
        assert (comment.externalId == "Foo2")
        assert (comment.externalType == "Bar2")
        _list_equal(comment.metadata, [1, 2, 3, 4])

        res.delete_comment(comments.data.comments[0])
        res.delete_comment(comments.data.comments[1])

        comments = res.list_comments()
        assert (len(comments.data.comments) == 0)

        # Now we handle res2
        comments = res2.list_comments()
        assert (len(comments.data.comments) == 1)
        comment = comments.data.comments[0]
        assert (comment.externalId == "Foo")
        assert (comment.externalType == "Bar")
        _list_equal(comment.metadata, [1, 2])
        res.delete_comment(comments.data.comments[0])
        comments = res.list_comments()
        assert (len(comments.data.comments) == 0)


def test_task_comment_feedback_reporting():
    """
    We want to be able to generate reports like this:

    Select Across Gorup    -- externalGroup
    Inputs Seen: XXX       -- Distinct externalId
    Inputs Suggested: YYY  -- Add to metadata
    Inputs Liked / Disliked / Used -- Add to metadata

    So really we just need to test the group aggregation
    """
    steamship = _steamship()
    name = _random_name()
    embedder = PluginInstance.create(steamship, pluginHandle='test-embedder').data
    with _random_index(steamship, pluginInstance=embedder.handle) as index:
        item1 = EmbeddedItem(
            value="Pizza",
            externalId="pizza",
            externalType="food",
            metadata=[1, 2, 3]
        )

        G1 = _random_name()
        G2 = _random_name()

        index.insert(item1.value, externalId=item1.externalId, externalType=item1.externalType, metadata=item1.metadata)
        task = index.embed()
        task.wait()

        res = index.search(item1.value, includeMetadata=True, k=1)
        res.add_comment(externalId="Foo1", externalType="Bar1", externalGroup=G1, metadata=[1, 2, 3])
        res.add_comment(externalId="Foo2", externalType="Bar1", externalGroup=G1, metadata=[1, 2, 3])
        res.add_comment(externalId="Foo2", externalType="Bar1", externalGroup=G2, metadata=[1, 2, 3])

        comments = res.list_comments()
        assert (len(comments.data.comments) == 3)

        g1 = steamship.tasks.list_comments(externalGroup=G1)
        assert (len(g1.data.comments) == 2)

        g2 = steamship.tasks.list_comments(externalGroup=G2)
        assert (len(g2.data.comments) == 1)

        g1 = steamship.tasks.list_comments(taskId=res.task.taskId, externalGroup=G1)
        assert (len(g1.data.comments) == 2)

        g2 = steamship.tasks.list_comments(taskId=res.task.taskId, externalGroup=G2)
        assert (len(g2.data.comments) == 1)

        g1 = steamship.tasks.list_comments(taskId=res.task.taskId, externalId="Foo1", externalGroup=G1)
        assert (len(g1.data.comments) == 1)

        g2 = steamship.tasks.list_comments(taskId=res.task.taskId, externalId="Foo1", externalGroup=G2)
        assert (len(g2.data.comments) == 0)

        res.delete_comment(comments.data.comments[0])
        res.delete_comment(comments.data.comments[1])
        res.delete_comment(comments.data.comments[2])

        g1 = steamship.tasks.list_comments(externalGroup=G1)
        assert (len(g1.data.comments) == 0)

        g2 = steamship.tasks.list_comments(externalGroup=G2)
        assert (len(g2.data.comments) == 0)



import io
import logging
import re
from dataclasses import dataclass
from typing import Tuple, Any, List

from steamship.base import Client, Response, Request
from steamship.base.binary_utils import flexi_create
from steamship.base.request import IdentifierRequest
from steamship.data.block import Block
from steamship.data.tags import Tag


class File:
    pass


class FileUploadType:
    file = "file"  # The CreateRequest contains a file upload that should be used
    url = "url"  # The CreateRequest contains a url that should be scraped
    value = "value"  # The Create Request contains a `text` field that should be used
    fileImporter = "fileImporter"  # The CreateRequest contains a fileImporter handle that should be used
    blocks = "blocks"  # The CreateRequest contains blocks and tags that should be read in directly


_logger = logging.getLogger(__name__)


def parseDquery(query: str) -> List[Tuple[str, str, str]]:
    query = re.sub(' +', ' ', query.strip())
    parts = re.split(r'\s*(?=[@#])', query)
    ret = []

    for s in parts:
        s = s.strip()
        if not s:
            continue

        command = ''
        if s[0] in ['@', '#']:
            command = s[0]
            s = s[1:]

        if command == '':
            ret.append((command, None, s))
            continue

        if '"' not in s and ":" not in s:
            if command == '#':
                ret.append((command, 'contains', s))
            else:
                ret.append((command, s, None))
            continue

        modifier = None
        if ':' in s:
            ss = s.split(':')
            modifier = ss[0]
            s = ss[1]

        content = s
        if '"' in s:
            i = s.index('"')
            content = s[1 + i:-1]
            if modifier is None:
                s = s[:i]
                modifier = s
                if modifier == '':
                    modifier = None

        ret.append((command, modifier, content))
    return ret


@dataclass
class FileClearResponse:
    id: str


@dataclass
class SpanQuery:
    text: str = None
    label: str = None
    spanType: str = None


@dataclass
class FileQueryRequest(Request):
    fileId: str
    type: str = None
    hasSpans: List[SpanQuery] = None
    text: str = None
    textMode: str = None
    isQuote: bool = None


@dataclass
class FileQueryResponse:
    id: str
    blocks: List[Block]

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "FileQueryResponse":
        return FileQueryResponse(
            id=d.get('id', None),
            blocks=[Block.from_dict(block, client=client) for block in d.get('blocks', None)]
        )


@dataclass
class File:
    """A file.
    """
    client: Client = None
    id: str = None
    name: str = None
    handle: str = None
    mimeType: str = None
    spaceId: str = None
    corpusId: str = None
    blocks: [Block] = None
    tags: [Tag] = None

    @dataclass
    class CreateRequest:
        value: str = None
        data: str = None
        url: str = None
        type: str = None  # FileUploadType: fileImporter | value | url | data

        mimeType: str = None
        corpusId: str = None
        name: str = None
        blocks: [Block.CreateRequest] = None
        tags: [Tag.CreateRequest] = None

        pluginInstance: str = None

        @staticmethod
        def from_dict(d: any, client: Client = None) -> "File.CreateRequest":
            return File.CreateRequest(
                value=d.get('value', None),
                data=d.get('data', None),
                url=d.get('url', None),
                type=d.get('type', None),
                mimeType=d.get('mimeType', None),
                corpusId=d.get('corpusId', None),
                pluginInstance=d.get('pluginInstance', None),
                name=d.get('name', None),
                blocks=[Block.CreateRequest.from_dict(block, client=client) for block in d.get('blocks', [])],
                tags=[Tag.CreateRequest.from_dict(tag, client=client) for tag in d.get('tags', [])]
            )

        def to_dict(self) -> dict:
            return dict(
                value=self.value,
                data=self.data,
                url=self.url,
                type=self.type,
                mimeType=self.mimeType,
                corpusId=self.corpusId,
                pluginInstance=self.pluginInstance,
                name=self.name,
                blocks=[block.to_dict() for block in self.blocks] if self.blocks else []
            )

    @dataclass
    class CreateResponse:
        data: Any = None
        mimeType: str = None

        def __init__(
                self,
                data: Any = None,
                string: str = None,
                bytes: io.BytesIO = None,
                json: io.BytesIO = None,
                mimeType: str = None
        ):
            data, mimeType = flexi_create(
                body=data,
                string=string,
                json=json,
                bytes=bytes,
                mimeType=mimeType
            )
            self.data = data
            self.mimeType = mimeType

        @staticmethod
        def from_dict(d: any, client: Client = None) -> "File.CreateResponse":
            return File.CreateResponse(
                data=d.get('data', None),
                mimeType=d.get('mimeType', None)
            )

        def to_dict(self) -> dict:
            return dict(
                data=self.data,
                mimeType=self.mimeType
            )

    @dataclass
    class ListRequest(Request):
        corpusId: str = None

    @dataclass
    class ListResponse:
        files: List["File"]

        @staticmethod
        def from_dict(d: any, client: Client = None) -> "File.ListResponse":
            return File.ListResponse(
                files=[File.from_dict(f, client=client) for f in d.get('files', [])]
            )

    @dataclass
    class RawRequest(Request):
        id: str

    @staticmethod
    def from_dict(d: any, client: Client = None) -> "File":
        if d is None:
            return None
        if 'file' in d:
            d = d['file']
        return File(
            client=client,
            id=d.get('id', None),
            handle=d.get('handle', None),
            name=d.get('name', None),
            mimeType=d.get('mimeType', None),
            corpusId=d.get('corpusId', None),
            spaceId=d.get('spaceId', None),
            blocks=[Block.from_dict(block, client=client) for block in d.get('blocks', [])],
            tags=[Tag.from_dict(tag, client=client) for tag in d.get('tags', [])]
        )

    def to_dict(self) -> dict:
        return dict(
            id=self.id,
            handle=self.handle,
            name= self.name,
            mimeType=self.mimeType,
            corpusId=self.corpusId,
            spaceId=self.spaceId,
            blocks=[block.to_dict() for block in self.blocks] if self.blocks else [],
            tags=[tag.to_dict() for tag in self.tags] if self.tags else []
        )


    def delete(
            self,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None) -> "Response[File]":
        return self.client.post(
            'file/delete',
            IdentifierRequest(id=self.id),
            expect=File,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def clear(
            self,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None) -> Response[FileClearResponse]:
        return self.client.post(
            'file/clear',
            IdentifierRequest(id=self.id),
            expect=FileClearResponse,
            ifdQuery=self,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    @staticmethod
    def get(
            client: Client,
            id: str = None,
            handle: str = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None) -> Response["File"]:
        return client.post(
            'file/get',
            IdentifierRequest(id=id, handle=handle),
            expect=File,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    @staticmethod
    def create(
            client: Client,
            filename: str = None,
            name: str = None,
            url: str = None,
            content: str = None,
            pluginInstance: str = None,
            mimeType: str = None,
            blocks: List[Block.CreateRequest] = None,
            tags: List[Tag.CreateRequest] = None,
            corpusId: str = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None
    ) -> "Response[File]":

        if filename is None and name is None and content is None and url is None and pluginInstance is None and blocks is None:
            raise Exception("Either filename, name + content, url, or plugin Instance must be provided.")

        uploadType = None
        if blocks is not None:
            uploadType = FileUploadType.blocks
        elif pluginInstance is not None:
            uploadType = FileUploadType.fileImporter
        elif content is not None:
            # We're still goign to use the file upload method for file uploads
            uploadType = FileUploadType.file
        elif filename is not None:
            with open(filename, 'rb') as f:
                content = f.read()
                name = filename
            uploadType = FileUploadType.file
        else:
            if url is not None:
                raise Exception("Unable to determine upload type. For scraping a URL, use the File.scrape method.")
            else:
                raise Exception("Unable to determine upload type.")

        req = File.CreateRequest(
            type=uploadType,
            corpusId=corpusId,
            name=name,
            url=url,
            mimeType=mimeType,
            pluginInstance=pluginInstance,
            blocks=blocks,
            tags=tags
        )

        return client.post(
            'file/create',
            payload=req,
            file=(name, content, "multipart/form-data") if uploadType != FileUploadType.blocks else None,
            expect=File,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    @staticmethod
    def list(
            client: Client,
            corpusId: str = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None
    ):
        req = File.ListRequest(
            corpusId=corpusId
        )
        res = client.post(
            'file/list',
            payload=req,
            expect=File.ListResponse,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )
        return res

    @staticmethod
    def scrape(
            client: Client,
            url: str,
            name: str = None,
            corpusId: str = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None) -> "File":
        if name is None:
            name = url
        req = File.CreateRequest(
            type=FileUploadType.url,
            name=name,
            url=url,
            corpusId=corpusId
        )

        return client.post(
            'file/create',
            payload=req,
            expect=File,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def dquery(
            self,
            dQuery: str,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None):
        blockType = None
        hasSpans = []
        text = None
        isQuote = None
        textMode = None

        for tup in parseDquery(dQuery):
            (cmd, subcmd, content) = tup
            if cmd == '':
                blockType = content
            elif cmd == '#':
                text = content
                textMode = subcmd
            elif cmd == '@':
                hasSpans.append(SpanQuery(label=subcmd, text=content))

        return self.query(
            blockType=blockType,
            hasSpans=hasSpans,
            text=text,
            textMode=textMode,
            isQuote=isQuote,
            pd=True,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    def query(
            self,
            hasSpans: List[SpanQuery] = None,
            text: str = None,
            textMode: str = None,
            isQuote: bool = None,
            pd: bool = False,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None
    ) -> Response[FileQueryResponse]:

        req = FileQueryRequest(
            fileId=self.id,
            hasSpans=hasSpans,
            text=text,
            textMode=textMode,
            isQuote=isQuote
        )
        res = self.client.post(
            'file/query',
            payload=req,
            expect=FileQueryResponse,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )
        return res

    def raw(
            self,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None):
        req = File.RawRequest(
            id=self.id,
        )

        return self.client.post(
            'file/raw',
            payload=req,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space,
            rawResponse=True
        )



from steamship import DocTag
from steamship.extension.file import File

from ..client.helpers import deploy_plugin, _steamship

__copyright__ = "Steamship"
__license__ = "MIT"


def test_e2e_converter():
    client = _steamship()
    with deploy_plugin("plugin_converter.py", "converter") as (plugin, version, instance):
        file = File.create(client=client, name="Test.txt", content="This is a test.").data
        assert (len(file.query().data.blocks) == 0)

        # Use the plugin we just registered
        file.convert(pluginInstance=instance.handle).wait()
        assert (len(file.query().data.blocks) == 4)

        file.delete()



from dataclasses import dataclass
from typing import List, Union

from steamship.base import Client, Request, Response
from steamship.base.request import IdentifierRequest
from steamship.data.tags.tag import Tag


@dataclass
class Block:
    client: Client = None
    id: str = None
    fileId: str = None
    text: str = None
    tags: List[Tag] = None

    @dataclass
    class CreateRequest(Request):
        id: str = None
        fileId: str = None
        text: str = None
        tags: List[Tag.CreateRequest] = None
        upsert: bool = None

        @staticmethod
        def from_dict(d: any, client: Client = None) -> "Block.CreateRequest":
            return Block.CreateRequest(
                id=d.get('id', None),
                fileId=d.get('fileId', None),
                text=d.get('text', None),
                tags=[Tag.CreateRequest.from_dict(tag, client=client) for tag in d.get("tags", [])],
                upsert=d.get('upsert', None),
            )

        def to_dict(self)-> dict:
            return dict(
                id=self.id,
                fileId=self.fileId,
                text=self.text,
                upsert=self.upsert,
                tags=[tag.to_dict() for tag in self.tags] if self.tags else []
            )

    @dataclass
    class DeleteRequest(Request):
        id: str = None

    @dataclass
    class ListRequest(Request):
        fileId: str = None

    @dataclass
    class ListResponse(Request):
        blocks: List["Block"] = None

        @staticmethod
        def from_dict(d: any, client: Client = None) -> "Block.ListResponse":
            if d is None:
                return None
            return Block.ListResponse(
                blocks=[Block.from_dict(x, client=client) for x in (d.get("blocks", []) or [])]
            )

    @staticmethod
    def from_dict(d: any, client: Client = None) -> Union["Block", None]:
        if d is None:
            return None
        return Block(
            client=client,
            id=d.get('id', None),
            fileId=d.get('fileId', None),
            text=d.get('text', None),
            tags=list(map(lambda tag: Tag.from_dict(tag, client=client), d.get('tags', [])))
        )

    def to_dict(self) -> dict:
        tags = None
        if self.tags is not None:
            tags = [tag.to_dict() for tag in self.tags]

        return dict(
            id=self.id,
            fileId=self.fileId,
            text=self.text,
            tags=tags
        )

    @staticmethod
    def get(
            client: Client,
            id: str = None,
            spaceId: str = None,
            spaceHandle: str = None,
            space: any = None) -> Response["Block"]:
        return client.post(
            'block/get',
            IdentifierRequest(id=id),
            expect=Block,
            spaceId=spaceId,
            spaceHandle=spaceHandle,
            space=space
        )

    @staticmethod
    def create(
            client: Client,
            fileId: str = None,
            text: str = None,
            tags: List[Tag.CreateRequest] = None,
            upsert: bool = None,
            spaceId: str = None,
            spaceHandle: str = None
    ) -> Response["Block"]:
        req = Block.CreateRequest(
            fileId=fileId,
            text=text,
            tags=tags,
            upsert=upsert
        )
        return client.post(
            'block/create',
            req,
            expect=Block,
            spaceId=spaceId,
            spaceHandle=spaceHandle
        )

    @staticmethod
    def listPublic(
            client: Client,
            fileId: str = None,
            spaceId: str = None,
            spaceHandle: str = None
    ) -> Response["Block.ListResponse"]:
        return client.post(
            'block/list',
            Block.ListRequest(
                fileId=fileId
            ),
            expect=Block.ListResponse,
            spaceId=spaceId,
            spaceHandle=spaceHandle
        )

    def delete(self) -> Response["Block"]:
        return self.client.post(
            'block/delete',
            Block.DeleteRequest(id=self.id),
            expect=Tag,
        )



from steamship import PluginInstance
from tests.client.helpers import _steamship

__copyright__ = "Steamship"
__license__ = "MIT"




def test_parsing():
    steamship = _steamship()
    parser = PluginInstance.create(steamship, pluginHandle='test-tagger').data
    resp = steamship.tag("This is a test", pluginInstance=parser.handle)
    resp.wait()
    resp = resp.data
    assert (len(resp.file.blocks) == 1)
    d = resp.file.blocks[0]

    assert (d.text == "This is a test")
    assert (len(d.tags) == 5)


# def test_ner():
#     steamship = _steamship()
#     resp = steamship.parse(["I like Ted"], plugin=parsing_plugin(), includeEntities=True).data
#     assert(len(resp.docs) == 1)
#     d = resp.docs[0]
#     assert(len(d.entities) == 1)
#     assert(d.entities[0].text == "Ted")

#     assert(len(d.sentences) == 1)
#     s = d.sentences[0]
#     assert(len(s.entities) == 1)
#     assert(s.entities[0].text == "Ted")


# def test_token_matcher():
#     steamship = _steamship()

#     a_matcher = TokenMatcher(
#       label="A_MATCHER",
#       patterns=[
#         [{"LOWER": "a"}, {"LOWER": "matcher"}]
#       ]
#     )

#     b_matcher = TokenMatcher(
#       label="B_MATCHER",
#       patterns=[
#         [{"LOWER": "see"}],
#         [{"LOWER": "if"}, {"LOWER": "a"}]
#       ]
#     )

#     resp = steamship.parse(
#       ["Let's see if a matcher works."], 
#       plugin=parsing_plugin(),      
#       tokenMatchers=[a_matcher, b_matcher]
#     ).data
#     assert(len(resp.docs) == 1)
#     d = resp.docs[0]
#     assert(len(d.sentences) == 1)
#     s = d.sentences[0]
#     assert(len(d.spans) == 3)
#     ans = {
#       "a matcher": a_matcher.label,
#       "see": b_matcher.label,
#       "if a": b_matcher.label
#     }
#     for sp in d.spans:
#       assert(sp.text in ans)
#       assert(sp.label == ans[sp.text])

# def test_phrase_matcher():
#     steamship = _steamship()

#     a_matcher = PhraseMatcher(
#       label="A_MATCHER",
#       phrases=["a matcher"]
#     )

#     b_matcher = PhraseMatcher(
#       label="B_MATCHER",
#       phrases=["if a", "see"]
#     )

#     resp = steamship.parse(
#       ["Let's see if a matcher works."], 
#       plugin=parsing_plugin(),      
#       phraseMatchers=[a_matcher, b_matcher]
#     ).data
#     assert(len(resp.docs) == 1)
#     d = resp.docs[0]
#     assert(len(d.sentences) == 1)
#     s = d.sentences[0]
#     assert(len(d.spans) == 3)
#     ans = {
#       "a matcher": a_matcher.label,
#       "see": b_matcher.label,
#       "if a": b_matcher.label
#     }
#     for sp in d.spans:
#       assert(sp.text in ans)
#       assert(sp.label == ans[sp.text])

# def test_phrase_matcher_attr():
#     steamship = _steamship()

#     a_matcher = PhraseMatcher(
#       label="IP_ADDR",
#       phrases=["127.0.0.1", "127.127.0.0"],
#       attr="SHAPE"
#     )

#     resp = steamship.parse(
#       ["Often the router will have an IP address such as 192.168.1.1 or 192.168.2.1."],
#       plugin=parsing_plugin(),      
#       phraseMatchers=[a_matcher]
#     ).data
#     assert(len(resp.docs) == 1)
#     d = resp.docs[0]
#     assert(len(d.sentences) == 1)
#     s = d.sentences[0]
#     assert(len(d.spans) == 2)
#     ans = {
#       "192.168.1.1": a_matcher.label,
#       "192.168.2.1": a_matcher.label
#     }
#     for sp in d.spans:
#       assert(sp.text in ans)
#       assert(sp.label == ans[sp.text])

# def test_token_phrase_matcher_combo():
#     steamship = _steamship()

#     a_matcher = PhraseMatcher(
#       label="A_MATCHER",
#       phrases=["a Matcher"]
#     )

#     b_matcher = PhraseMatcher(
#       label="B_MATCHER",
#       phrases=["if a", "see"]
#     )

#     c_matcher = PhraseMatcher(
#       label="IP_ADDR",
#       phrases=["127.0.0.1", "127.127.0.0", "88.88.88.88"],
#       attr="SHAPE"
#     )

#     d_matcher = TokenMatcher(
#       label="FOO",
#       patterns = [[{"LOWER": "can"}]]
#     )

#     e_matcher = PhraseMatcher(
#       label="CAP",
#       phrases=["Ted"],
#       attr="SHAPE"
#     )

#     f_matcher = PhraseMatcher(
#       label="accordingly",
#       phrases=["accordingly"],
#       attr="LEMMA"
#     )

#     resp = steamship.parse(
#       ["Let's see if a Matcher can match 44.33.22.11 accordingly."], 
#       plugin=parsing_plugin(),      
#       tokenMatchers=[d_matcher],
#       phraseMatchers=[a_matcher, b_matcher, c_matcher, e_matcher, f_matcher]
#     ).data

#     assert(len(resp.docs) == 1)
#     d = resp.docs[0]
#     assert(len(d.sentences) == 1)
#     s = d.sentences[0]
#     assert(len(d.spans) == 7)
#     ans = {
#       "a Matcher": a_matcher.label,
#       "see": b_matcher.label,
#       "if a": b_matcher.label,
#       "can": d_matcher.label,
#       "44.33.22.11": c_matcher.label,
#       "Matcher": e_matcher.label,
#       "f_matcher": f_matcher.label,
#       "Let": e_matcher.label,
#       "accordingly": f_matcher.label
#     }
#     for sp in d.spans:
#       assert(sp.text in ans)
#       assert(sp.label == ans[sp.text])

# def test_dependency_matcher():
#     steamship = _steamship()

#     a_matcher = DependencyMatcher(
#       label="FOUNDED",
#       patterns=[
#         [
#           {
#             "RIGHT_ID": "anchor_founded",       # unique name
#             "RIGHT_ATTRS": {"ORTH": "founded"}  # token pattern for "founded"
#           }
#         ]
#       ]
#     )

#     resp = steamship.parse(
#       ["Smith founded two companies"],
#       plugin=parsing_plugin(),      
#       dependencyMatchers=[a_matcher]
#     ).data
#     assert(len(resp.docs) == 1)
#     d = resp.docs[0]
#     assert(len(d.sentences) == 1)
#     s = d.sentences[0]
#     assert(len(d.spans) == 1)
#     ans = {
#       "founded": a_matcher.label
#     }
#     for sp in d.spans:
#       assert(sp.text in ans)
#       assert(sp.label == ans[sp.text])

#     a_matcher = DependencyMatcher(
#       label="FOUNDED",
#       patterns=[
#         [
#           {
#             "RIGHT_ID": "anchor_founded",       # unique name
#             "RIGHT_ATTRS": {"ORTH": "founded"},  # token pattern for "founded"
#             "LABEL": "FOUND"
#           }
#         ]
#       ]
#     )

#     resp = steamship.parse(
#       ["Smith founded two companies"],
#       plugin=parsing_plugin(),      
#       dependencyMatchers=[a_matcher]
#     ).data
#     assert(len(resp.docs) == 1)
#     d = resp.docs[0]
#     assert(len(d.sentences) == 1)
#     s = d.sentences[0]
#     assert(len(d.spans) == 2)
#     ans = {
#       "founded": a_matcher.label
#     }
#     for sp in d.spans:
#       assert(sp.text in ans)
#       assert(sp.label in ["FOUNDED", "FOUND"])


#     pattern = [
#         {
#             "RIGHT_ID": "anchor_founded",
#             "RIGHT_ATTRS": {"ORTH": "founded"},
#             "LABEL": "ROOT"
#         },
#         {
#             "LEFT_ID": "anchor_founded",
#             "REL_OP": ">",
#             "RIGHT_ID": "founded_subject",
#             "RIGHT_ATTRS": {"DEP": "nsubj"},
#             "LABEL": "WHO"
#         },
#         {
#             "LEFT_ID": "anchor_founded",
#             "REL_OP": ">",
#             "RIGHT_ID": "founded_object",
#             "RIGHT_ATTRS": {"DEP": "dobj"},
#             "LABEL": "WHAT"
#         },
#         {
#             "LEFT_ID": "founded_object",
#             "REL_OP": ">",
#             "RIGHT_ID": "founded_object_modifier",
#             "RIGHT_ATTRS": {"DEP": {"IN": ["amod", "compound"]}},
#         }
#     ]
#     sent = "Lee, an experienced CEO, has founded two AI startups."

#     a_matcher = DependencyMatcher(
#       label="FOUNDED",
#       patterns=[pattern]
#     )
#     resp = steamship.parse(
#       [sent],
#       plugin=parsing_plugin(),      
#       dependencyMatchers=[a_matcher]
#     ).data

#     assert(len(resp.docs) == 1)
#     d = resp.docs[0]
#     assert(len(d.sentences) == 1)
#     s = d.sentences[0]
#     assert(len(d.spans) == 4)
#     spans = d.spans
#     ans = {
#       "startups": "WHAT",
#       "Lee": "WHO",
#       "founded": "ROOT",
#       "Lee, an experienced CEO, has founded two AI startups": "FOUNDED"
#     }
#     assert(len(spans) == 4)
#     for sp in spans:
#       assert(sp.text in ans)
#       assert(sp.label == ans[sp.text])



from steamship import Block, DocTag, Tag
from steamship.app import App, post, create_handler
from steamship.plugin.tagger import Tagger
from steamship.plugin.service import PluginResponse, PluginRequest
from steamship.plugin.inputs.block_and_tag_plugin_input import BlockAndTagPluginInput
from steamship.plugin.outputs.block_and_tag_plugin_output import BlockAndTagPluginOutput
import re



def tagSentences(block: Block):
    """Splits the document into sentences by assuming a period is a sentence divider."""
    # Add the period back
    tags = []
    for m in re.finditer(r'[^\.]+', block.text):
        tags.append(Tag(kind=DocTag.doc, name=DocTag.sentence, startIdx=m.start(), endIdx=m.end()+1))
    if block.tags:
        block.tags.extend(tags)
    else:
        block.tags = tags



def _makeTestResponse(request: BlockAndTagPluginInput) -> BlockAndTagPluginOutput:
    for block in request.file.blocks:
        tagSentences(block)
    response = BlockAndTagPluginOutput(request.file)
    return response


class TestParserPlugin(Tagger, App):
    # TODO: WARNING! We will need to implement some logic that prevents 
    # a distributed endless loop. E.g., a parser plugin returning the results
    # of using the Steamship client to call parse.. via itself!

    def run(self, request: PluginRequest[BlockAndTagPluginInput]) -> PluginResponse[BlockAndTagPluginOutput]:
        if request.data is not None:
            return PluginResponse(
                data=_makeTestResponse(request.data)
            )

    @post('tag')
    def parse(self, **kwargs) -> dict:
        parseRequest = Tagger.parse_request(request=kwargs)
        parseResponse = self.run(parseRequest)
        return Tagger.response_to_dict(parseResponse)


handler = create_handler(TestParserPlugin)
